<?php

namespace Drupal\tmgmt_cdt\Entity;

use Drupal\Core\Language\Language;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\tmgmt\Entity\Job;
use Drupal\tmgmt\JobInterface;
use Drupal\tmgmt\JobItemInterface;
use Drupal\tmgmt\TMGMTException;
use Drupal\tmgmt_cdt\TmgmtCdtHelper;
use Drupal\Component\Serialization\Json;

/**
 * Override class Job.
 */
class TmgmtCdtJob extends Job
{

    use StringTranslationTrait;

    /**
     * {@inheritdoc}
     */
    public function label($langcode = null)
    {
        $label = parent::label($langcode);
        if (empty($label)) {
            return 'Translation job ' . (string) $this->get('tjid')->value;
        } else {
            return $label;
        }

    }

    /**
     * {@inheritdoc}
     */
    public function getConflictingItems()
    {
        return array();
    }

    /**
     * Get cdt setting.
     */
    public function getCdtSetting($name, $default = false)
    {
        $cdt_settings = (array) @Json::decode($this->get('cdt_settings')->value);
        if (isset($cdt_settings[$name])) {
            return $cdt_settings[$name];
        } else {
            return $default;
        }
    }

    /**
     * Set cdt setting.
     */
    public function setCdtSetting($name, $data)
    {
        $cdt_settings = (array) @Json::decode($this->get('cdt_settings')->value);
        $cdt_settings[$name] = $data;
        $this->set('cdt_settings', Json::encode($cdt_settings));
    }

    /**
     * Create all job items.
     */
    public function jobItemGenerateTargetLanguages()
    {

        $target_languages = array_values($this->getSetting('cdt')['target_languages']);
        foreach ($this->getItems() as $job_item) {

            for ($i = 0; $i < \count($target_languages); $i++) {
                if ($i === 0) {
                    // First lang for the current item.
                    $job_item->set('cdt_target_language', $target_languages[$i]);
                    $job_item->save();
                } else {
                    // Create duplicates items for other languages.
                    $clone_job_item = [];
                    $clone_job_item['tjid'] = $job_item->tjid;
                    $clone_job_item['data'] = $job_item->get('data')->value;
                    $clone_job_item['cdt_source_language'] = $job_item->get('cdt_source_language')->value;
                    $clone_job_item['cdt_target_language'] = $target_languages[$i];
                    $clone_job_item['cdt_character_count'] = $job_item->get('cdt_character_count')->value;
                    $clone_job_item['cdt_settings'] = $job_item->get('cdt_settings')->value;
                    $clone_job_item['state'] = JobItemInterface::STATE_ACTIVE;

                    $item = tmgmt_job_item_create($job_item->plugin, $job_item->get('item_type')->value, $job_item->get('item_id')->value, $clone_job_item);

                    $item->save();
                }
            }
        }
    }

    /**
     * If has items with translations to review.
     */
    public function hasItemsToReview()
    {
        $needReview = false;
        foreach ($this->getItems() as $job_item) {
            if ($job_item->isNeedsReview()) {
                $needReview = true;
                break;
            }
        }
        return $needReview;
    }

    /**
     * Adds an item to the translation job.
     */
    public function addItem($plugin, $item_type, $item_id, $fields = array())
    {

        $transaction = \Drupal::database()->startTransaction();
        $is_new = false;

        if ($this->isNew()) {
            $this->save();
            $is_new = true;
        }

        $fields['tjid'] = $this->id();
        $fields['cdt_source_language'] = $this->get('source_language')->value;

        $item = tmgmt_job_item_create(
            $plugin, $item_type, $item_id, $fields
        );

        $character_count = $item->translatableCharacterCount();

        if ($character_count === 0) {
            $transaction->rollback();

            if ($is_new) {
                $this->tjid = null;
            }

            throw new TMGMTException(
                'Job item @label (@type) has no translatable content.',
                ['@label' => $item->label(), '@type' => $item->getSourceType()]
            );
        }

        $item->set('cdt_character_count', $character_count);
        $item->save();

        return $item;

    }

    /**
     * {@inheritdoc}
     */
    public function addTranslatedDataLanguage(array $data, $target_language)
    {
        $job_items = $this->getItemsLanguage($target_language);
        $count = 0;

        foreach ($job_items as $job_item) {
            if ($job_item->isNeedsReview()) {
                continue;
            }
            $count++; 
            foreach ($data as $key => $value) {
                if ($job_item->get('item_id')->value == $key) {
                    $job_item->addTranslatedData($value);
                }
            }
        }

        return $count;
    }

  /**
   * Update job items status on new pickup.
   */
  public function pickupPrepareJobItems() {

    $cdt_settings = @Json::decode($this->get('cdt_settings')->value);
    
    if ($this->getCdtSetting('on_pickup', FALSE)) {
      return;
    }

    foreach ($this->getItems() as $job_item) {
      if ($job_item->isNeedsReview()) {
        $job_item->active();
        $job_item->save();
      }
    }

    $this->setCdtSetting('on_pickup', TRUE);
    $this->save();

  }

    /**
     * Same as parent::getItems but with language.
     */
    public function getItemsLanguage($target_language)
    {

        $items = [];
        $query = \Drupal::entityQuery('tmgmt_job_item')
            ->condition('tjid', $this->id())
            ->condition('cdt_target_language', $target_language);

        $results = $query->execute();
        if (!empty($results)) {
            $items = \Drupal::entityTypeManager()->getStorage('tmgmt_job_item')->loadMultiple($results);
        }
        return $items;
    }

    /**
     * {@inheritdoc}
     */
    public function getTargetLanguage()
    {
        if ($this->get('target_language') === 'multiple') {
            $lang = [
                'name' => 'Multiple',
                'id' => 'multiple',
                'direction' => 'ltr',
                'locked' => false,
            ];
            return new Language($lang);
        } else {
            return \Drupal::languageManager()->getDefaultLanguage();
        }

    }

    /**
     * Add more indicators.
     */
    public function getCounts()
    {
        $count = [];
        foreach ($this->getItems() as $item) {
            $target_language = $item->get('cdt_target_language')->value;
            if (!$target_language) {
                $target_language = 'empty';
            }
            if (!isset($count[$target_language][$item->get('item_id')->value])) {
                $count[$target_language][$item->get('item_id')->value] = 0;
            }
            //$item->recalculateStatistics();
            $count[$target_language][$item->get('item_id')->value] += $item->get('cdt_character_count')->value;
        }

        if (count($count) > 0) {
            $languages = count(array_keys($count));
            $characters_language = (int) array_sum(reset($count));
            $pages_language = ceil($characters_language / 750) * 0.5;
            $items = count(reset($count));
        } else {
            $languages = 0;
            $characters_language = 0;
            $pages_language = 0;
            $items = 0;
        }

        $out = [
            'languages' => $languages,
            'characters' => $characters_language * $languages,
            'characters_language' => $characters_language,
            'items' => $items,
            'pages' => $pages_language * $languages,
            'pages_language' => $pages_language,
        ];

        return $out;
    }

    /**
     * {@inheritdoc}
     */
    public function isAbortable()
    {
        return false;
    }

    /**
     * {@inheritdoc}
     */
    public function acceptTranslation()
    {
        $items_max = (int) TmgmtCdtHelper::getTranslatorSetting('conf_advanced.job_max_process');
        $items_count = 0;

        $job_counts = $this->getCounts();
        $total_items = (int) $job_counts['items'] * (int) $job_counts['languages'];
        unset($job_counts);

        $options = [
            'state' => JobItemInterface::STATE_REVIEW,
        ];
        foreach ($this->getItems($options) as $item) {

            if ($items_count >= $items_max) {
                \Drupal::messenger()->addWarning($this->t('This job is big (@total_items items), @items_count done, please do it again for the remaining items',
                    [
                        '@total_items' => $total_items,
                        '@items_count' => $items_count,
                    ]));
                break;
            }
            if ($item->acceptTranslation()) {
                $items_count++;
            }
        }

        if (tmgmt_job_check_finished($this->id())) {
            $this->setState(JobInterface::STATE_FINISHED);
            $this->set('target_language', 'multiple');
            $this->save();
            $this->addMessage($this->t('The translation job has been finished.'));
        }
    }

    /**
     * {@inheritdoc}
     */
    public function getSourceLangcode()
    {
        return \Drupal::languageManager()->getDefaultLanguage()->getId();
    }

 
}

/**
 * To use JobItem->getSourceData() from items in cart.
 *
 * They are not yet linked to a parent job.
 */
class FakeTmgmtCdtJob
{

    /**
     * {@inheritdoc}
     */
    public function getSourceLangcode()
    {
        return \Drupal::languageManager()->getDefaultLanguage()->getId();
    }

    /**
     * {@inheritdoc}
     */
    public function isActive()
    {
        return false;
    }

}
