<?php

namespace Drupal\tmgmt_cdt;

use Drupal\Component\Serialization\Json;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Site\Settings;
use Drupal\tmgmt_cdt\Entity\TmgmtCdtJob;
use Drupal\tmgmt_cdt\Entity\TmgmtCdtJobItem;
use Drupal\tmgmt_cdt\TmgmtCdtHelper;

/**
 * Commands class.
 */
class TmgmtCdtCommands
{

    /**
     * process a command.
     */
    public static function command($content)
    {
        try {

            $command = trim((string) @$content['command']);
            $data = (array) @$content['data'];

            if (empty($command)) {
                throw new \Exception('Command is empty');
            }

            if (!method_exists(static::class, $command)) {
                throw new \Exception('Unknown command: ' . $command);
            }

            self::$command($data);

        } catch (\Exception $e) {
            \Drupal::messenger()->addError($e->getMessage());
            return false;
        }
    }

    /**
     * Call several commands in sequence.
     */
    public static function multiCommand($data = array())
    {
        $commands = (array) @$data;

        $count = 0;
        foreach ($commands as $content) {
            self::command($content);
            $count++;
            \Drupal::messenger()->addStatus(t('@count - command <strong>@command</strong> executed.', array('@command' => @$content['command'], '@count' => $count)));
        }

    }

    /**
     * Export proxy parameters to Logger/Messenger.
     */
    public static function proxySettings($data = array())
    {

        $proxy = Settings::get('http_client_config', 'No proxy settings');
        switch (@$data['mode']) {
            case 'logger':
                \Drupal::logger('tmgmt_cdt')->debug('http_client_config: <pre><code>' . var_export($proxy, true) . '</code></pre>');
                \Drupal::messenger()->addWarning(t('http_client_config settings exported to Drupal logger.'));
                break;

            case 'messenger':
            default:
                \Drupal::messenger()->addStatus(var_export($proxy, true));
                break;
        }
    }

    /**
     * reset settings to default values.
     */
    public static function resetSettings($data = array())
    {

        $config = \Drupal::service('config.factory')->getEditable('tmgmt.translator.tmgmt_cdt');

        foreach (TmgmtCdtHelper::defaultSettings() as $key => $value) {
            $config->set('settings.' . $key, $value);
        }
        $config->save();

        \Drupal::state()->set('tmgmt_cdt.ready', false);
        \Drupal::messenger()->addStatus(t('All options have been reset. Please restart the checklist.'));
    }

    /**
     *  Generate a global report.
     */
    public static function report($data = array())
    {

        $report = array();

        $version_current = @TmgmtCdtHelper::getState('version')['current']['version'] ? TmgmtCdtHelper::getState('version')['current']['version'] : '?';

        $report['viewer'] = 'jsoneditoronline.org';
        $report['generated'] = date("j F Y, g:i a");
        $report['drupal version'] = \DRUPAL::VERSION;
        $report['tmgmt module version'] = @\Drupal::service('extension.list.module')->getExtensionInfo('tmgmt')['version'];
        $report['tmgmt_cdt module version'] = @\Drupal::service('extension.list.module')->getExtensionInfo('tmgmt_cdt')['version'];
        $report['tmgmt_cdt internal version'] = $version_current;
        $report['tmgmt_cdt is ready'] = TmgmtCdtHelper::getState('ready');
        $report['tmgmt.translator.tmgmt_cdt'] = \Drupal::config('tmgmt.translator.tmgmt_cdt')->get();

        $report['tmgmt_job count'] = \count((array) \Drupal::entityTypeManager()->getStorage('tmgmt_job')->getQuery()->execute());
        $report['tmgmt_job_item count'] = \count((array) \Drupal::entityTypeManager()->getStorage('tmgmt_job_item')->getQuery()->execute());
        $report['tmgmt_message count'] = \count((array) \Drupal::entityTypeManager()->getStorage('tmgmt_message')->getQuery()->execute());

        $report['languages'] = array();
        $default = \Drupal::languageManager()->getDefaultLanguage();
        $report['languages']['default'] = array(
            'name' => $default->getName(),
            'id' => $default->getId(),
            'weight' => $default->getWeight(),
        );
        unset($default);
        $list = \Drupal::languageManager()->getLanguages();
        $native_list = \Drupal::languageManager()->getStandardLanguageList();
        $report['languages']['list'] = array();
        foreach ($list as $id => $language) {
            $report['languages']['list'][$id] = array(
                'name' => $language->getName(),
                'native' => @$native_list[$id][1],
                'id' => $language->getId(),
                'weight' => $language->getWeight(),
                'default' => $language->isDefault(),
                'direction' => $language->getDirection(),
            );
        }
        unset($list);
        $report['modules'] = array_keys(\Drupal::moduleHandler()->getModuleList());
        $report['version'] = TmgmtCdtHelper::getState('version');
        $report['credentials'] = TmgmtCdtHelper::getCredentials();
        $report['states'] = \Drupal::state()->getMultiple(array_keys(TmgmtCdtHelper::defaultStates()));

        $file_name = 'tmgmt_cdt_report.json';
        $directory = 'public://tmgmt_file';
        \Drupal::service('file_system')->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS);
        $file = file_save_data(Json::encode($report), "public://tmgmt_file/$file_name", FileSystemInterface::EXISTS_REPLACE);
        if (!$file) {
            throw new \Exception('Cannot create file');
        }
        \Drupal::service('file.usage')->add($file, 'tmgmt_cdt', 'report', 'tmgmt_cdt', 0);

        \Drupal::messenger()->addStatus(t('Report file created, can be downloaded <a href="@link" download>here</a>.',
            ['@link' => file_create_url("public://tmgmt_file/$file_name")]));

    }

    /**
     * Update a job state.
     */
    public static function jobState($data = array())
    {

        $job_id = (int) $data['job_id'];
        $new_state_id = (int) $data['new_state_id'];

        $job = TmgmtCdtJob::load($job_id);

        if (!$job) {
            throw new \Exception('Cannot load job id: ' . $job_id);
        }

        if (!isset(TmgmtCdtJob::getStates()[$state_id])) {
            throw new \Exception('Unknown state id: ' . $new_state_id);
        }
        $state_id = $job->setState($new_state_id, t('Change state by command.'));
        $job->save();

        \Drupal::messenger()->addStatus(t('Job id (@job_id) now has state: @state.',
            array('@job_id' => $job_id,
                '@state' => TmgmtCdtJob::getStates()[$state_id]))
        );

    }

    /**
     * Update a job item state.
     */
    public static function jobItemState($data = array())
    {

        $job_item_id = (int) $data['job_item_id'];
        $new_state_id = (int) $data['new_state_id'];

        $job_item = TmgmtCdtJobItem::load($job_item_id);

        if (!$job_item) {
            throw new \Exception('Cannot load job item id: ' . $job_item_id);
        }

        if (!isset(TmgmtCdtJobItem::getStates()[$new_state_id])) {
            throw new \Exception('Unknown state id: ' . $new_state_id);
        }

        $state_id = $job_item->setState($new_state_id, t('Change state by command.'));
        $job_item->save();

        \Drupal::messenger()->addStatus(t('Job item id (@job_item_id) now has state: @state.',
            array('@job_item_id' => $job_item_id,
                '@state' => TmgmtCdtJobItem::getStates()[$state_id]))
        );

    }

    /**
     * Update state of multiple job items.
     */
    public static function multiJobItemState($data = array())
    {

        $job_item_ids = (array) @$data['job_item_ids'];
        $job_id = (int) @$data['job_id'];
        $new_state_id = (int) $data['new_state_id'];

        if ($job_id > 0) {
            $job = TmgmtCdtJob::load($job_id);
            if (!$job) {
                throw new \Exception('Cannot load job id: ' . $job_id);
            }
            $job_item_ids = array_keys($job->getItems());
        }

        $count = 0;
        foreach ($job_item_ids as $job_item_id) {
            self::jobItemState(array(
                'job_item_id' => $job_item_id,
                'new_state_id' => $new_state_id,
            ));
            $count++;
        }

        \Drupal::messenger()->addStatus(t('@count Job items has now state: @state.',
            array('@count' => $count,
                '@state' => TmgmtCdtJobItem::getStates()[$new_state_id]))
        );

    }

    /**
     * Report a job.
     */
    public static function jobReport($data = array())
    {

        $job_id = (int) $data['job_id'];
        $include_job_items = @$data['include_job_items'];
        $include_messages = @$data['include_messages'];

        $job = TmgmtCdtJob::load($job_id);
        if (!$job) {
            throw new \Exception('Cannot load job id: ' . $job_id);
        }

        $report = array();
        $report['viewer'] = 'jsoneditoronline.org';
        $report['generated'] = date("j F Y, g:i a");
        $report['job'] = array();

        foreach (array_keys($job->getFields()) as $field) {
            $report['job'][$field] = $job->get($field)->value;
        }
        $states = TmgmtCdtJob::getStates();
        $report['job']['state'] = $states[$report['job']['state']];
        $report['job']['created'] = date(\DateTimeInterface::ATOM, $report['job']['created']);
        $report['job']['changed'] = date(\DateTimeInterface::ATOM, $report['job']['changed']);

        $report['job']['items count'] = count($job->getItems());

        if ($include_job_items) {
            $report['job']['items'] = array();
            $states = TmgmtCdtJobItem::getStates();
            foreach ($job->getItems() as $item) {
                $data = array();
                foreach (array_keys($item->getFields()) as $field) {
                    $data[$field] = $item->get($field)->value;
                }
                $data['state'] = $states[$data['state']];
                $data['changed'] = date(\DateTimeInterface::ATOM, $data['changed']);

                $report['job']['items'][] = $data;
                $data = null;
            }
        }

        if ($include_messages) {
            $report['job']['messages'] = array();
            foreach ($job->getMessages() as $message) {
                $report['job']['messages'][] = $message->getMessage()->render();
            }
        }

        $file_name = 'tmgmt_cdt_report_job.json';
        $directory = 'public://tmgmt_file';
        \Drupal::service('file_system')->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS);
        $file = file_save_data(Json::encode($report), "public://tmgmt_file/$file_name", FileSystemInterface::EXISTS_REPLACE);
        if (!$file) {
            throw new \Exception('Cannot create file');
        }
        \Drupal::service('file.usage')->add($file, 'tmgmt_cdt', 'report', 'tmgmt_cdt', 0);

        \Drupal::messenger()->addStatus(t('Job report file created, can be downloaded <a href="@link" download>here</a>.',
            ['@link' => file_create_url("public://tmgmt_file/$file_name")]));

    }

    /**
     * Update a job fields.
     */
    public static function jobUpdate($data = array())
    {

        $job_id = (int) $data['job_id'];
        $fields = (array) $data['fields'];

        $job = TmgmtCdtJob::load($job_id);

        if (!$job) {
            throw new \Exception('Cannot load job id: ' . $job_id);
        }

        foreach ($fields as $field => $data) {
            $job->set($field, $data);
        }
        $job->save();

        \Drupal::messenger()->addStatus(t('Job id (@job_id) updated.',
            array('@job_id' => $job_id))
        );

    }

    /**
     * Update a job item fields.
     */
    public static function jobItemUpdate($data = array())
    {

        $job_item_id = (int) $data['job_item_id'];
        $fields = (array) $data['fields'];

        $job_item = TmgmtCdtJobItem::load($job_item_id);

        if (!$job_item) {
            throw new \Exception('Cannot load job item id: ' . $job_item_id);
        }

        foreach ($fields as $field => $data) {
            $job_item->set($field, $data);
        }
        $job_item->save();

        \Drupal::messenger()->addStatus(t('Job item id (@job_item_id) updated.',
            array('@job_item_id' => $job_item_id))
        );

    }

    /**
     * Update multiple job items fields.
     */
    public static function multiJobItemUpdate($data = array())
    {
        $job_item_ids = (array) @$data['job_item_ids'];
        $job_id = (int) @$data['job_id'];
        $fields = (array) $data['fields'];

        if ($job_id > 0) {
            $job = TmgmtCdtJob::load($job_id);
            if (!$job) {
                throw new \Exception('Cannot load job id: ' . $job_id);
            }
            $job_item_ids = array_keys($job->getItems());
        }

        $count = 0;
        foreach ($job_item_ids as $job_item_id) {
            self::jobItemUpdate(array(
                'job_item_id' => $job_item_id,
                'fields' => $fields,
            ));
            $count++;
        }

        \Drupal::messenger()->addStatus(t('@count Job items updated.',
            array('@count' => $count))
        );

    }

    /**
     * Install and setup chosen.
     */
    public static function setupChosen($data = array())
    {

        $chosen_installed = in_array('chosen', array_keys(\Drupal::moduleHandler()->getModuleList()));
        if (!$chosen_installed) {
            $installed = \Drupal::service('module_installer')->install(['chosen'], true);
            if (!$installed) {
                \Drupal::messenger()->addError(t('Cannot install the module chosen. Check your error logs.'));
                return;
            }
            \Drupal::messenger()->addWarning(t('Module <strong>chosen</strong> has been installed.'));
        } else {
            \Drupal::messenger()->addStatus(t('Module <strong>chosen</strong> already intalled.'));
        }

        $config = \Drupal::service('config.factory')->getEditable('chosen.settings');

        if (count($config->getOriginal()) === 0) {
            \Drupal::messenger()->addError(t('Cannot read configuration <strong>chosen.settings</strong>.'));
            return;
        }

        $config->set('minimum_multiple', 0); // Minimum number of options for multi select: Always apply
        $config->set('minimum_width', 250); // Minimum width of widget: 250px
        $config->set('jquery_selector', 'select[multiple="multiple"]'); // Apply Chosen to the following elements: select[multiple="multiple"]
        $config->set('chosen_include', 0); // Include Chosen only on admin pages

        $config->save();
        \Drupal::messenger()->addStatus(t('Configuration updated.'));

    }

    /**
     * Clean old tmgmt_cdt setup.
     */
    public static function cleanSetup($data = array())
    {

        \Drupal::state()->deleteMultiple(TmgmtCdtHelper::oldStates());

        \Drupal::messenger()->addStatus(t('@count Job items updated.',
            array('@count' => $count))
        );

    }

    /**
     * Get and store last version current, release ans dev.
     */
    public static function updateVersion($data = array())
    {
        $version = TmgmtCdtHelper::updateVersion();
        \Drupal::messenger()->addWarning(t('Updated with @value.', array(
            '@value' => var_export($version, true),
        )));
    }

    /**
     * Update config tmgmt.translator.tmgmt_cdt.
     */
    public static function updateProvider($data = array())
    {

        $config = \Drupal::service('config.factory')->getEditable('tmgmt.translator.tmgmt_cdt');

        foreach ($data as $key => $value) {
            $config->set($key, $value);
        }
        $config->save();

        \Drupal::messenger()->addStatus(t('Provider tmgmt_cdt updated.'));

    }

}
