<?php

namespace Drupal\tmgmt_cdt;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Render\Element;
use Drupal\tmgmt\JobItemInterface;
use Drupal\tmgmt_cdt\Entity\TmgmtCdtJobItem;
use Drupal\tmgmt_cdt\TmgmtCdtHelper;

/**
 * Helper.
 */
class TmgmtCdtFormatHelper
{

    const DEFAULT = 'Default';
    const CUSTOM = 'Custom';
  
    /**
     * Protect specials contents inside inside {{ cdt-xxx: }} tags to avoid translation.
     *
     * @param string $html
     * @return string
     */
    public static function convertDrupalToCdt($html)
    {
        try {
            if (!(bool) TmgmtCdtHelper::getTranslatorSetting('conf_features.convert_content')) {
                return $html;
            }
            $html = self::convertHtmlScript($html);
            $html = self::convertDrupalToken($html);
            $html = self::convertDrupalTwig($html);
            $html = self::convertDrupalCustom($html);

            return $html;
        } catch (\Exception $e) {
            \Drupal::messenger()->addError($e->getMessage());
            return $html;
        }
    }

    /**
     * Detect field translations settings.
     */
    public static function jobItemFieldsTranslationDetection(TmgmtCdtJobItem $job_item)
    {

        $job_item->onCart = true;
        $data = $job_item->getSourceData();

        $item = array();
        $translate_settings = $job_item->getCdtSetting('translate_settings');
        $item['translation_type'] = isset($translate_settings['translation_type']) ? $translate_settings['translation_type'] : self::default;

        $item['fields'] = array();

        foreach (Element::children($data) as $key) {
            $element = \Drupal::service('tmgmt.data')->flatten($data[$key], $key);
            if ($element) {
                foreach (Element::children($element) as $_key) {
                    $data_item = $element[$_key];

                    if (isset($data_item['#text']) && \Drupal::service('tmgmt.data')->filterData($data_item)) {
                        $field = array();

                        $field['field_name'] = str_replace('][', '|', $_key);
                        $field['full_field_name'] = $job_item->get('item_type')->value . '][' . $_key;
                        $field['text'] = $data_item['#text'];
                        $field['in_exclude_list'] = TmgmtCdtHelper::isFieldExcluded($field['full_field_name']);
                        $field['character_count'] = self::characterCount($field['text']);
                        $field['content_to_translate'] = $field['character_count'] > 0;
                        $content_converted = self::convertDrupalToCdt($field['text']);
                        $field['content_is_converted'] = $content_converted !== $field['text'];
                        $field['content_is_converted'] ? $field['converted_content'] = $content_converted : $field['converted_content'] = '';

                        if ($item['translation_type'] === self::CUSTOM) {
                            $field['translation_settings'] = $translate_settings['fields'][$field['field_name']];
                        }

                        $item['fields'][$field['field_name']] = $field;
                    }

                }

            }

        }

        return $item;

    }

    /**
     * Restore original html from translated content.
     *
     * @param string $html
     * @return string
     */
    public static function convertCdtToDrupal($html)
    {
        try {

            preg_match_all('/tmgmt-cdt-(.+?)="(.+?)"|<tmgmt-cdt type="(.+?)">(.+?)<\/tmgmt-cdt>/s', $html, $matches, PREG_SET_ORDER, 0);

            foreach ($matches as $value) {
                $converted = $value[0];
                $type = substr($converted, 0, 10);

                switch ($type) {
                    case 'tmgmt-cdt-':
                        // type attribute
                        $html = str_replace($converted, substr($converted, 10), $html);
                        break;
                    case '<tmgmt-cdt':
                        // type tag
                        $mode = $value[3];
                        $content = $value[4];
                        switch ($mode) {
                            case 'reserved':
                                $html = str_replace($converted, $content, $html);
                                break;
                            case 'base64':
                                $html = str_replace($converted, base64_decode($content), $html);
                                break;
                            default:
                                throw new Exception("convertCdtToDrupal, Unknown mode: $mode");
                                break;
                        }
                        break;
                    default:
                        throw new Exception("convertCdtToDrupal, Unknown type: $type");
                        break;
                }
            }

            return $html;
        } catch (\Exception $e) {
            \Drupal::messenger()->addError($e->getMessage());
            return $html;
        }
    }

    /**
     * Protect drupal tokens [..] contents inside custom cdt tag/attribute.
     *
     * @param string $html
     * @return string
     */
    public static function convertDrupalToken($html)
    {
        if (!(bool) TmgmtCdtHelper::getTranslatorSetting('conf_features.convert_drupal_token')) {
            return $html;
        }
        try {
            // Simple token [...]. // TODO dexclude token with html tags inside [] '<'

            preg_match_all('/(\S+?)="([^"]*)\[.+?\]([^"]*)"|(\[.+?\])/s', $html, $matches, PREG_OFFSET_CAPTURE | PREG_SET_ORDER, 0);
            $live_offset = 0;
            foreach ($matches as $find) {
                if ($find[1][0] !== "") {
                    // inside attribute
                    $original = $find[0][0]; // aaa="bbbb"
                    $replace = 'tmgmt-cdt-' . $original;
                    $position = $find[0][1];
                    $html = substr_replace($html, $replace, $position + $live_offset, strlen($original));
                    $live_offset += 10;
                } else {
                    // outside attribute
                    $original = $find[0][0];
                    $replace = '<tmgmt-cdt type="reserved">' . $original . '</tmgmt-cdt>';
                    $position = $find[0][1];
                    $html = substr_replace($html, $replace, $position + $live_offset, strlen($original));
                    $live_offset += 39;
                }
            }
            return $html;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Extracts tmgmt-cdt-* attributs with tags as prefix, ex 'a.tmgmt-cdt-title'.
     */
    public static function convertedAttributs($html)
    {
        $dom = new DOMDocument();
        $list = array();
        $dom->loadHTML($html, LIBXML_HTML_NOIMPLIED);
        $elements = $dom->getElementsByTagName('*');

        foreach ($elements as $child) {
            foreach ($child->attributes as $attr) {
                if (substr($attr->nodeName, 0, 10) === 'tmgmt-cdt-') {
                    array_push($list, $child->nodeName . '.' . $attr->nodeName);
                }
            }
        }
        return array_unique($list);
    }

    /**
     * Protect drupal twig { .. } contents inside custom cdt tag/attribute.
     *
     * @param string $html
     * @return string
     */
    public static function convertDrupalTwig($html)
    {
        if (!(bool) TmgmtCdtHelper::getTranslatorSetting('conf_features.convert_drupal_twig')) {
            return $html;
        }
        try {
            // Simple token [...].
            preg_match_all('/(\S+?)="[^"]*{{.+?}}[^"]*"|{{.+?}}/s', $html, $matches, PREG_OFFSET_CAPTURE | PREG_SET_ORDER, 0);
            $live_offset = 0;
            foreach ($matches as $find) {
                if ($find[1][0] !== "") {
                    // inside attribute
                    $original = $find[0][0]; // aaa="bbbb"
                    $replace = 'tmgmt-cdt-' . $original;
                    $position = $find[0][1];

                    $html = substr_replace($html, $replace, $position + $live_offset, strlen($original));
                    $live_offset += 10;
                } else {
                    // outside attribute
                    $original = $find[0][0];
                    $replace = '<tmgmt-cdt type="reserved">' . $original . '</tmgmt-cdt>';
                    $position = $find[0][1];
                    $html = substr_replace($html, $replace, $position + $live_offset, strlen($original));
                    $live_offset += 39;
                }
            }
            return $html;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Hide drupal custom [..] contents inside custom cdt tag.
     *
     * @param string $html
     * @return string
     */
    public static function convertDrupalCustom($html)
    {
        if (!(bool) TmgmtCdtHelper::getTranslatorSetting('conf_features.convert_custom')) {
            return $html;
        }
        try {
            $keywords = TmgmtCdtHelper::getTranslatorSetting('conf_advanced.custom_keywords');
            $regex = '(' . str_replace(';', '|', $keywords) . ')';
            //return preg_replace($regex, '<tmgmt-cdt type="reserved">$0</tmgmt-cdt>', $html);
            preg_match_all('/(\S+?)="[^"]*' . $regex . '[^"]*"|' . $regex . '/s', $html, $matches, PREG_OFFSET_CAPTURE | PREG_SET_ORDER, 0);
            $live_offset = 0;
            foreach ($matches as $find) {
                if ($find[1][0] !== "") {
                    // inside attribute
                    $original = $find[0][0]; // aaa="bbbb"
                    $replace = 'tmgmt-cdt-' . $original;
                    $position = $find[0][1];

                    $html = substr_replace($html, $replace, $position + $live_offset, strlen($original));
                    $live_offset += 10;
                } else {
                    // outside attribute
                    $original = $find[0][0];
                    $replace = '<tmgmt-cdt type="reserved">' . $original . '</tmgmt-cdt>';
                    $position = $find[0][1];
                    $html = substr_replace($html, $replace, $position + $live_offset, strlen($original));
                    $live_offset += 39;
                }
            }
            return $html;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Hide html script tags <script> contents inside custom cdt tag.
     *
     * @param string $html
     * @return string
     */
    public static function convertHtmlScript($html)
    {
        if (!(bool) TmgmtCdtHelper::getTranslatorSetting('conf_features.convert_html_script')) {
            return $html;
        }
        try {
            $doc = new \DomDocument('1.0', 'UTF-8');
            libxml_use_internal_errors(true);

            $doc->loadHTML($html);
            $tags = $doc->getElementsByTagName("script");
            foreach ($tags as $tag) {
                $original = $doc->saveHTML($tag);
                $encoded = '<tmgmt-cdt type="base64">' . base64_encode($original) . '</tmgmt-cdt>';
                $html = str_replace($original, $encoded, $html);
            }
            return $html;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Check if html or not.
     */
    public static function isHtml($text)
    {
        $raw_text = \strip_tags($text);
        if ($raw_text !== $text) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Get html tags count.
     *
     * @param [type] $html
     * @return int
     */
    public static function tagsCount($html)
    {
        preg_match_all('/<((?=!\-\-)!\-\-[\s\S]*\-\-|((?=\?)\?[\s\S]*\?|((?=\/)\/[^.\-\d][^\/\]\'"[!#$%&()*+,;<=>?@^`{|}~ ]*|[^.\-\d][^\/\]\'"[!#$%&()*+,;<=>?@^`{|}~ ]*(?:\s[^.\-\d][^\/\]\'"[!#$%&()*+,;<=>?@^`{|}~ ]*(?:=(?:"[^"]*"|\'[^\']*\'|[^\'"<\s]*))?)*)\s?\/?))>/mi', $html, $matchs);
        return \count($matchs[0]);
    }

    /**
     * Get html tags count by type.
     *
     * @param string $html
     * @return array
     */
    public static function tagsTypeCount($html)
    {
        preg_match_all('/<((?=!\-\-)!\-\-[\s\S]*\-\-|((?=\?)\?[\s\S]*\?|((?=\/)\/[^.\-\d][^\/\]\'"[!#$%&()*+,;<=>?@^`{|}~ ]*|[^.\-\d][^\/\]\'"[!#$%&()*+,;<=>?@^`{|}~ ]*(?:\s[^.\-\d][^\/\]\'"[!#$%&()*+,;<=>?@^`{|}~ ]*(?:=(?:"[^"]*"|\'[^\']*\'|[^\'"<\s]*))?)*)\s?\/?))>/mi', $html, $matchs);
        $list = [];
        foreach (array_values($matchs[1]) as $val) {
            $exp = explode(' ', mb_strtolower($val));
            if (!isset($list[$exp[0]])) {
                $list[$exp[0]] = 0;
            }
            $list[$exp[0]]++;
        }
        return $list;
    }

    /**
     * get reference link and title for a drupal content.
     *
     * @param JobItemInterface $job_item
     * @return array
     */
    public static function itemSourceReference(JobItemInterface $job_item)
    {

        $item = [
            'link' => '',
            'label' => '',
        ];

        switch ($job_item->get('item_type')->value) {
            case 'node':
            case 'taxonomy_term':
                $entity = \Drupal::entityTypeManager()->getStorage($job_item->get('item_type')->value)->load($job_item->get('item_id')->value);
                $item['link'] = $entity->toUrl('canonical', ['absolute' => true])->toString();
                $item['label'] = $entity->label();
                break;
            default:
                $item['link'] = \Drupal::request()->getSchemeAndHttpHost();
                $item['label'] = \Drupal::config('system.site')->get('name');
        }

        return $item;
    }

    /**
     * Character count with CdT rules.
     *
     * @param string $text original content
     * @param boolean $convert use cdt conversion
     * @param string $text_clean clean content for testing
     * @return int
     */
    public static function characterCount($text, $convert = true, &$text_clean = '')
    {
        $text = \strip_tags($text); // remove html tags
        $text = preg_replace('/{{.*?}}/m', '', $text); // remove twig tags
        $text = trim(str_replace(' ', '', $text)); // remove spaces
        $text = preg_replace('/\x{00A0}|&nbsp;|&#160;|&#xa0;|\s/', '', $text); //remove spaces
        if ($convert) {
            $text = self::convertDrupalToCdt($text); // protect special content
            //$text = preg_replace('/{{.*?}}/m', '', $text); // remove cdt tags
            $text = preg_replace('/<tmgmt-cdt.*?tmgmt-cdt>/m', '', $text); // remove cdt tags
        }
        $text_clean = $text;
        return mb_strlen(trim($text));
    }

    /**
     * Extract urls (href).
     *
     * @param html $html
     * @return array
     */
    public static function extractHrefs($html)
    {

        $hrefs = [];
        preg_match_all('/<a[\s]+([^>]+)>((?:.(?!\<\/a\>))*.)<\/a>/im', $html, $matchs, PREG_SET_ORDER, 0);

        foreach ($matchs as $id => $item) {
            preg_match_all('/href=[\"\'](.*?)[\"\']/im', $item[0], $href, PREG_SET_ORDER, 0);
            if (empty($href)) {
                continue;
            }
            $hrefs[] = [
                // Full tag.
                'a' => $item[0],
                'label' => $item[2],
                'href' => isset($href[0][1]) ? $href[0][1] : '',
            ];
        }
        unset($id);
        return $hrefs;
    }

    /**
     * Detect converted attributs not yet managed by TAO.
     */
    public static function hasNewAttributesConverted($job)
    {

        try {
            // get rules from git dev branch to have the last version
            $curl_options = array('CURLOPT_USERAGENT' => 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/101.0.0.0 Safari/537.36');

            $branch_dev = self::curlGetFileContents('https://git.drupalcode.org/api/v4/projects/61708/repository/branches/8.x-2.x', $curl_options);
            if (!$branch) {
                throw new Exception('Cannot get module last updates');
            }
            $branch_dev = @Json::decode($branch_dev);
            $commit = $branch_dev['commit']['id'];

            $version = self::curlGetFileContents('https://git.drupalcode.org/project/tmgmt_cdt/-/raw/' . $commit . '/json/version.json', $curl_options);
            $version = @Json::decode($version);

            $current_list = @$version['filters']['attributs'];

            $items = array();
            foreach ($job->getItems() as $item) {
                // TODO ne garder que les unique  type+id array_push
                // in_array

            }

            // compare with attributs converted on this job

            // if new attribut find, generate a message for TAO on job request

        } catch (\Exception $e) {
            \Drupal::logger('tmgmt_cdt')->error($e->getMessage());
            return false;
        }

    }

}
