<?php

namespace Drupal\tmgmt_cdt;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Site\Settings;
use Drupal\tmgmt\Entity\Translator;
use Drupal\tmgmt\JobInterface;
use Drupal\tmgmt_cdt\Entity\TmgmtCdtJob;
use Drupal\tmgmt_cdt\Entity\TmgmtCdtJobItem;

/**
 * Helper class.
 */
class TmgmtCdtHelper
{

    const SIMULATION = 'SIMULATION';
    const PRODUCTION = 'PRODUCTION';

    const NAME = 'Translation Centre (CdT)';
    const DEFAULT_REQUEST_IDENTIFIER = '0000/unknown';

    const API_STATUS_COMPLETED = 'COMPLETED';
    const API_STATUS_QUOTED_PENDING_APPROVAL = 'QUOTED - PENDING APPROVAL';
    const API_STATUS_CANCELLED = 'CANCELLED';

    const API_COMMENT_URLS = [
        'DO_NOT_ADAPT_EXTERNAL_URLS' => 'Do not adapt external urls',
        'ADAPT_EXTERNAL_URLS' => 'Adapt external urls',
    ];

    const API_URL = [
        'ACCEPTANCE' => 'https://b2becdt-a.cdt.europa.eu/generic-rest/',
        'PRODUCTION' => 'https://b2becdt.cdt.europa.eu/generic-rest/',
    ];

    const FIELDS_TRANSLATION_MODE = [
        'EXCLUSION_LIST' => 'Disable only the fields present in the exclusion list',
        'AUTO' => 'Automatically detect fields to exclude',
        'ALL' => 'Exclude nothing',
    ];

    /**
     * Check if is a json.
     */
    public static function isJson($data)
    {
        // @codingStandardsIgnoreStart
        try {
            // must be \json_decode function.
            if (\is_string($data) && \is_array(\json_decode($data, true)) && (json_last_error() === JSON_ERROR_NONE)) {
                return true;
            } else {
                return false;
            }
        } catch (\Exception $e) {
            return false;
        }
        // @codingStandardsIgnoreEnd
    }

    /**
     * Get default options.
     */
    public static function defaultStates()
    {
        $states = [
            'tmgmt_cdt.ready' => false,
            'tmgmt_cdt.business_reference_data' => new \stdClass(),
            'tmgmt_cdt.api_url' => '',
            'tmgmt_cdt.api_username' => '',
            'tmgmt_cdt.api_password' => '',
            'tmgmt_cdt.xsd_source' => '',
            'tmgmt_cdt.api_access_token_expire' => 0,
            'tmgmt_cdt.api_access_token' => '',
            'tmgmt_cdt.version' => '',
        ];

        return $states;
    }

    /**
     * Check job status, and update if needed.
     */
    public static function jobAutoUpdate(TmgmtCdtJob $job)
    {

        if ($job->isFinished()) {
            return $job;
        }

        if (($job->get('cdt_request_identifier')->value === TmgmtCdtHelper::DEFAULT_REQUEST_IDENTIFIER) || empty($job->get('cdt_request_identifier')->value)) {
            $api = TmgmtCdtApi::getInstance();
            $request_identifier = $api->getRequestIdentifier($job->get('cdt_correlation_id')->value);

            if ($request_identifier !== false) {
                $job->addMessage('Reference (Request Identifier) updated');
                $job->set('cdt_request_identifier', trim($request_identifier, '"'));
                $job->set('reference', trim($request_identifier, '"'));
                $job->set('target_language', 'multiple');
                $job->save();
            } else {
                return $job;
            }
        }

        $api = TmgmtCdtApi::getInstance();
        $api->setParamsPath(TmgmtCdtHelper::splitRequestIdentifier($job->get('cdt_request_identifier')->value));
        $api->get('requests/{requestyear}/{requestnumber}');

        if ($api->response->status === 'success' && ($api->response->content->status === TmgmtCdtHelper::API_STATUS_CANCELLED)) {
            $job->setState(JobInterface::STATE_ABORTED, t('Updated from webservice'));
            \Drupal::messenger()->addWarning(t('Job "@label" has been cancelled', ['@label' => $job->label()]));
            $job->save();
        }

        if (!$job->isFinished()) {
            if (tmgmt_job_check_finished($job->id())) {
                $job->setState(JobInterface::STATE_FINISHED);
                $job->set('target_language', 'multiple');
                $job->save();
                $job->addMessage(t('The translation job has been finished.'));
            }
        }

        return $job;
    }

    /**
     * Pull translations from CdT.
     */
    public static function pullTranslations(TmgmtCdtJob $job)
    {
        try {

            $cdt_request_identifier = trim($job->get('cdt_request_identifier')->value);

            if (empty($cdt_request_identifier)) {
                throw new \Exception('Request_identifier missing');
            }

            $api = TmgmtCdtApi::getInstance();
            $is_complete = $api->isTranslationComplete(TmgmtCdtHelper::splitRequestIdentifier($cdt_request_identifier));

            if (!$is_complete) {
                \Drupal::messenger()->addWarning(t('Translations are not yet available'));
            } else {

                $api = TmgmtCdtApi::getInstance();
                $translations = $api->getTranslations(TmgmtCdtHelper::splitRequestIdentifier($cdt_request_identifier));

                if (!$translations) {
                    throw new \Exception(implode(', ', $api->response->errors));
                }
                unset($api);

                $translatorPlugin = $job->getTranslatorPlugin();
                $translator = $job->getTranslator();
                $translatorPlugin->validateRequestResponse($job, $translations);

                $errors = 0;
                $job_items_max = (int) TmgmtCdtHelper::getTranslatorSetting('conf_advanced.job_max_process');
                $job_items_count = 0;

                $job_counts = $job->getCounts();
                $total_items = (int) $job_counts['items'] * (int) $job_counts['languages'];
                unset($job_counts);

                $job->pickupPrepareJobItems();

                $is_full_done = true;
                foreach ($translations as $target) {

                    $target_language_remote = $target->targetLanguage;
                    $target_language_local = array_flip($translator->getRemoteLanguagesMappings())[$target_language_remote];

                    $conditions = [
                        'state' =>
                        [
                            'operator' => 'IN',
                            'value' => [JobItemInterface::STATE_ACTIVE,
                                JobItemInterface::STATE_REVIEW,
                            ],
                        ],
                        'cdt_target_language' => $target_language_local,
                    ];

                    $job_items_target = $job->getItems($conditions);

                    if (count($job_items_target) === 0) {
                        continue;
                    }
                    if ($job_items_count > $job_items_max) {
                        $errors++;
                        $is_full_done = false;
                        \Drupal::messenger()->addWarning(t('This job is big (@total_items items), @job_items_count done, please do it again for the remaining items',
                            [
                                '@total_items' => $total_items,
                                '@job_items_count' => $job_items_count,
                            ]));
                        break;
                    }

                    $xml_data_target = base64_decode($target->fileBase64);

                    $xml_data_source = \file_get_contents(\Drupal::service('file_system')->realpath('public://tmgmt_file/cdt_job_' . $job->id() . '_request.xml'));
                    if (!$xml_data_source) {
                        $errors++;
                        \Drupal::messenger()->addError(t('Cannot read the file: public://tmgmt_file/cdt_job_@job_id_request.xml to validate target @target_language',
                            [
                                '@job_id' => $job->id(),
                                '@target_language' => $target_language_remote,
                            ]));
                        continue;
                    }

                    $available_languages = tmgmt_available_languages([\Drupal::languageManager()->getDefaultLanguage()->getId()]);
                    if (!in_array($target_language_local, array_keys($available_languages), true)) {
                        $errors++;
                        \Drupal::messenger()->addError(t('Invalid target language @target_language', ['@target_language' => $target_language_remote]));
                        continue;
                    }

                    $file_format = new TmgmtCdtXml();
                    $validate = $file_format->validateImport($xml_data_target);
                    if ($validate !== true) {
                        $errors++;
                        \Drupal::messenger()->addError(t('Cannot validate import for language @target_language: @error',
                            [
                                '@target_language' => $target_language_local,
                                '@error' => $validate,
                            ]));
                        continue;
                    }
                    $translated_data = $file_format->importXml($job->id(), $xml_data_target, $target_language_local);
                    if (!empty($translated_data)) {

                        $job_items_done = $job->addTranslatedDataLanguage($translated_data, $target_language_local);

                        if ($job_items_done > 0) {
                            $job_items_count = $job_items_count + $job_items_done;
                        }
                    } else {
                        $errors++;
                        \Drupal::messenger()->addWarning(t('Translations for language @target_language are empty.', ['@target_language' => $target_language_local]));
                    }

                }

                if ($errors === 0) {
                    $message = t('Translations have been imported from @name. You can now review the content', ['@name' => TmgmtCdtHelper::NAME]);
                    \Drupal::messenger()->addStatus($message);
                } else {
                    $message = t('Translations have been imported from @name but with some warnings, please check the messages.', ['@name' => TmgmtCdtHelper::NAME]);
                    \Drupal::messenger()->addWarning($message);
                }
                $job->addMessage($message);

                if ($is_full_done) {
                    $job->setCdtSetting('on_pickup', false);
                    $job->set('cdt_response', \json_encode($translations));
                    $job->set('target_language', 'multiple');
                    $job->save();
                } else {
                    $job->set('target_language', 'multiple');
                }
                $job->save();

            }
        } catch (\Exception $e) {
            // to be sur we can restart a new pickup
            @$job->setCdtSetting('on_pickup', false);
            @$job->save();
            \Drupal::messenger()->addError($e->getMessage());

        }
    }

    /**
     * Get default options.
     */
    public static function defaultSettings($onlyPrefix = false)
    {

        if ($onlyPrefix) {
            return [
                'conf_mode',
                'conf_features',
                'conf_advanced',
                'conf_network',
            ];
        }

        $settings = [
            'conf_mode.api_mode' => self::SIMULATION,

            'conf_network.curl_proxy_host' => '',
            'conf_network.curl_proxy_port' => '',
            'conf_network.curl_proxy_user' => '',
            'conf_network.curl_proxy_pass' => '',
            'conf_network.curl_proxy_user_agent' => '',
            'conf_network.curl_options' => '{"CURLOPT_TIMEOUT":15,"CURLOPT_RETURNTRANSFER":true,"CURLOPT_SSL_VERIFYHOST":0,"CURLOPT_SSL_VERIFYPEER":0}',

            'conf_features.convert_content' => true,
            'conf_features.convert_html_script' => true,
            'conf_features.convert_drupal_token' => true,
            'conf_features.convert_drupal_twig' => true,
            'conf_features.convert_custom' => true,
            'conf_features.html_tags_comparison' => true,
            'conf_features.show_quotation' => false,
            'conf_features.disable_xsd_validation' => false,
            'conf_features.debug' => false,
            'conf_features.disable_simulator_injection' => false,
            'conf_features.cart_field_convert' => false,
            'conf_features.fields_translation_mode' => 'AUTO',

            'conf_advanced.max_pages_per_request' => 10,
            'conf_advanced.custom_keywords' => '<none>;<front>;<separator>;<nolink>;<minipanel>;_default;_none',
            'conf_advanced.xsd_link' => 'http://static.cdt.europa.eu/webtranslations/schemas/Drupal-Translation-V8-2.xsd',
            'conf_advanced.exclude_fields' => 'view][label;view][description;view][display][*][display_options][fields]*[separator;view][display][*][display_options][pager][options][tags][next;view][display][*][display_options][pager][options][tags][previous',
            'conf_advanced.job_max_items' => 15,
            'conf_advanced.job_max_process' => 150,
        ];

        return $settings;
    }

    /**
     * Old states no longer used.
     */
    public static function oldStates()
    {

        $states = [

            'tmgmt_cdt.api_mode',
            'tmgmt_cdt.debug',

            'tmgmt_cdt.curl_proxy_host',
            'tmgmt_cdt.curl_proxy_port',
            'tmgmt_cdt.curl_proxy_user',
            'tmgmt_cdt.curl_proxy_pass',
            'tmgmt_cdt.curl_proxy_user_agent',
            'tmgmt_cdt.curl_options',
            'tmgmt_cdt.max_pages_per_request',

            'tmgmt_cdt.convert_content',
            'tmgmt_cdt.convert_html_script',
            'tmgmt_cdt.convert_drupal_token',
            'tmgmt_cdt.convert_drupal_twig',
            'tmgmt_cdt.convert_custom',
            'tmgmt_cdt.custom_keywords',

            'tmgmt_cdt.html_check',
            'tmgmt_cdt.show_quotation',
            'tmgmt_cdt.xsd_link',
            'tmgmt_cdt.disable_xsd_validation',

            'tmgmt_cdt.exclude_fields',
            'tmgmt_cdt.disable_simulator_injection',

            'tmgmt_cdt.job_max_items',
            'tmgmt_cdt.job_max_process',

            'tmgmt_cdt.formats_html',
            'tmgmt_cdt.use_token_drupal',
            'tmgmt_cdt.use_token_html_a',
            'tmgmt_cdt.use_token_html_img',
            'tmgmt_cdt.use_token_html_script',

        ];

        return $states;
    }

    /**
     * Get default options.
     */
    public static function getState($name)
    {
        try {
            if (!isset(self::defaultStates()["tmgmt_cdt.$name"])) {
                throw new \Exception("Unknown tmgmt_cdt.$name option");
            }
            return \Drupal::state()->get("tmgmt_cdt.$name", self::defaultStates()["tmgmt_cdt.$name"]);
        } catch (\Exception $e) {
            \Drupal::messenger()->addError($e->getMessage());
            return false;
        }
    }

    /**
     * Get default options.
     */
    public static function getTranslatorSetting($name)
    {
        try {
            $settings = \Drupal::config('tmgmt.translator.tmgmt_cdt')->get('settings');
            $name_parts = explode('.', $name);

            if (!isset($name_parts[1])) {
                // uniquement le prefix
                if (!in_array($name, self::defaultSettings(true))) {
                    throw new \Exception("Unknown $name setting");
                }
                return $settings[$name];
            } else {
                if (!isset(self::defaultSettings()[$name])) {
                    throw new \Exception("Unknown $name setting");
                }
                $setting = $settings[$name_parts[0]][$name_parts[1]];
                if ($setting === null) {
                    $setting = self::defaultSettings()[$name];
                }
                return $setting;
            }

        } catch (\Exception $e) {
            \Drupal::messenger()->addError($e->getMessage());
            return null;
        }
    }

    /**
     * File get content using translator proxy settings.
     */
    public static function curlGetFileContents($url, $options = [])
    {
        try {

            $proxy_host = trim(self::getTranslatorSetting('conf_network.curl_proxy_host'));
            $proxy_port = (int) self::getTranslatorSetting('conf_network.curl_proxy_port');
            $proxy_user = trim(self::getTranslatorSetting('conf_network.curl_proxy_user'));
            $proxy_pass = trim(self::getTranslatorSetting('conf_network.curl_proxy_pass'));
            $proxy_user_agent = trim(self::getTranslatorSetting('conf_network.curl_proxy_user_agent'));
            $curl_options = Json::decode(self::getTranslatorSetting('conf_network.curl_options'));

            $curl = curl_init();
            $curlopt[CURLOPT_RETURNTRANSFER] = 1;
            $curlopt[CURLOPT_URL] = $url;

            if (!empty($proxy_host)) {
                $curlopt[CURLOPT_PROXY] = $proxy_host;
                $curlopt[CURLOPT_PROXYPORT] = $proxy_port;
            }

            if (!empty($proxy_user)) {
                $curlopt[CURLOPT_PROXYUSERPWD] = $proxy_user . ':' . $proxy_pass;
            }

            if (!empty($proxy_user_agent)) {
                $curlopt[CURLOPT_USERAGENT] = $proxy_user_agent;
            }

            foreach ($curl_options as $curl_option => $curl_value) {
                $curlopt[constant($curl_option)] = $curl_value;
            }

            foreach ($options as $curl_option => $curl_value) {
                $curlopt[constant($curl_option)] = $curl_value;
            }

            curl_setopt_array($curl, $curlopt);
            $raw_data = curl_exec($curl);
            if (curl_errno($curl)) {
                throw new \Exception(curl_error($curl));
            }

            curl_close($curl);
            unset($curl);

            if ($raw_data) {
                return $raw_data;
            } else {
                return false;
            }

        } catch (\Exception $e) {
            \Drupal::messenger()->addError($e->getMessage());
            return false;
        }

    }

    /**
     * Show if the module status is ready or not.
     */
    public static function showStatus($showReady = false)
    {
        if ($showReady) {
            if (!(bool) self::getState('ready')) {
                \Drupal::messenger()->addWarning(t('<strong>@name</strong> translator is <strong>not ready</strong>.', ['@name' => self::NAME]));
            } else {
                \Drupal::messenger()->addStatus(t('<strong>@name</strong> translator is <strong>ready</strong>.', ['@name' => self::NAME]));
            }
        }

        if (self::getTranslatorSetting('conf_mode.api_mode') === self::SIMULATION && (bool) self::getState('ready')) {
            \Drupal::messenger()->addWarning(
                t(
                    '<strong>@name</strong> use the mode <strong>@mode</strong>.',
                    [
                        '@name' => self::NAME,
                        '@mode' => self::getTranslatorSetting('conf_mode.api_mode'),
                    ]
                )
            );
        }

    }

    /**
     * Get api credentials (user, pass and webservice url).
     */
    public static function getCredentials()
    {
        $credentials = Settings::get('tmgmt_cdt', false);
        if (!$credentials) {
            $credentials = [
                'api_url' => self::getState('api_url'),
                'api_username' => self::getState('api_username'),
                'api_password' => self::getState('api_password'),
                'source' => 'state', // from \Drupal::state
            ];
        } else {
            $credentials['source'] = 'settings'; // from settings.php
        }

        return $credentials;
    }

    /**
     * Extract year and number.
     */
    public static function splitRequestIdentifier(string $request_number)
    {
        $request_ids = explode('/', trim($request_number));
        $request_year = @trim($request_ids[0]);
        $request_number = @trim($request_ids[1]);
        return [
            'requestyear' => $request_year,
            'requestnumber' => $request_number,
        ];
    }

    /**
     * Delete array key by path.
     */
    public static function arrayDeleteByPath(array &$data, string $path = '', string $separator = '][')
    {
        $segments = explode($separator, $path);
        $size = count($segments);
        $cur = &$data;
        $count = 0;
        foreach ($segments as $segment) {
            $count++;
            if (!isset($cur[$segment])) {
                return null;
            }
            if ($count === $size) {
                if (isset($cur[$segment])) {
                    unset($cur[$segment]);
                    return null;
                }
            }
            $cur = &$cur[$segment];
        }
    }

    /**
     * Check job rules.
     */
    public static function checkJob($job)
    {
        $errors = [];
        try {
            $volume = (int) $job->getCounts()['pages_language'];
            $max = (int) self::getTranslatorSetting('conf_advanced.max_pages_per_request');
            if ($volume > $max) {
                $errors[] = t(
                    'The translation volume is too large (@volume pages). The maximum per request is @max pages.',
                    ['@volume' => $volume, '@max' => $max]
                );
            }

            return $errors;
        } catch (\Exception $e) {
            $errors[] = $e->getMessage();
            return $errors;
        }
    }

    /**
     * Check job items.
     */
    public static function checkCartItems(array $items)
    {
        $errors = array();

        $total_characters = 0;
        $total_pages = 0;
        $max_pages = (int) self::getTranslatorSetting('conf_advanced.max_pages_per_request');
        $max_items = (int) self::getTranslatorSetting('conf_advanced.job_max_items');
        $items_count = 0;

        $job_items = TmgmtCdtJobItem::loadMultiple($items);
        foreach ($job_items as $job_item) {
            $item_characters = 0;
            if (!$job_item) {
                $messages['errors'][] = t(
                    'Error while loading an item. Impossible to verify, check the list.'
                );
                continue;
            }
            $items_count++;

            $item_characters = $job_item->translatableCharacterCount();
            $total_characters += $item_characters;

            if ($item_characters === 0) {
                $errors[] = t(
                    'The <i>@label</i> item has no content to translate.',
                    ['@label' => $job_item->getSourceLabel()]
                );
            }
        }

        $total_pages = ceil($total_characters / 750) * 0.5;
        if ($total_pages > $max_pages) {
            $errors[] = t(
                'The translation volume is too large (@total_pages pages). The maximum per request is @max_pages pages.',
                ['@total_pages' => $total_pages, '@max_pages' => $max_pages]
            );
            $errors[] = t(
                'You can change the max value in the translation module options, please contact the @cdt to check your needs.',
                ['@cdt' => TmgmtCdtHelper::NAME]
            );
        }

        if ($items_count > $max_items) {
            $errors[] = t(
                'Too many items: @items_count. The maximum per request is @max_items items.',
                ['@items_count' => $items_count, '@max_items' => $max_items]
            );
            if ($total_pages <= $max_pages) {
                $errors[] = t(
                    'Your max page limit (@max_pages) is not reached (@total_pages). You can temporarily increase the max limit of items per job. Contact the @cdt to check your needs.',
                    ['@cdt' => TmgmtCdtHelper::NAME,
                        '@total_pages' => $total_pages,
                        '@max_pages' => $max_pages,
                    ]
                );
            }
        }
        return $errors;
    }

    /**
     * Return job by request_identifier.
     */
    public static function jobByRequestIdentifier(string $request_identifier)
    {
        if ($request_identifier === self::DEFAULT_REQUEST_IDENTIFIER) {
            return false;
        }

        $query = \Drupal::entityQuery('tmgmt_job')
            ->condition('translator', 'tmgmt_cdt')
            ->condition('cdt_request_identifier', $request_identifier);
        $result = $query->execute();
        if (empty($result)) {
            return false;
        }

        $ids = array_keys($result);
        $job = \Drupal::entityTypeManager()->getStorage('tmgmt_job')->load((int) reset($ids));

        return $job;
    }

    /**
     * Get fields list to exclude.
     */
    public static function getFieldsToExclude()
    {
        $fields_exclude_list = explode(';', trim(self::getTranslatorSetting('conf_advanced.exclude_fields')));
        // Clean items.
        for ($i = 0; $i < count($fields_exclude_list); $i++) {
            $field = strtolower(trim($fields_exclude_list[$i]));
            $fields_exclude_list[$i] = $field;
        }
        return $fields_exclude_list;
    }

    /**
     * Check field from the global exclude list.
     */
    public static function isFieldExcluded($field)
    {
        $is_excluded = false;
        $items = self::getFieldsToExclude();

        $addRegexLiteral = function ($text) {

            return '\Q' . $text . '\E';
        };

        foreach ($items as $item) {
            $e = explode('*', $item);
            $e2 = array_map($addRegexLiteral, $e);
            $regex = '/^' . implode('.*?', $e2) . '$/';
            preg_match_all($regex, trim($field), $matches, PREG_SET_ORDER, 0);
            if (count($matches) > 0) {
                $is_excluded = true;
                break;
            }
        }

        return $is_excluded;
    }

    /**
     * Get version curent, release and dev.
     */
    public static function updateVersion($only_current = false)
    {

        try {

            $curl_options = array('CURLOPT_USERAGENT' => 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/101.0.0.0 Safari/537.36');

            $version = array();
            //

            $current = @\file_get_contents(\Drupal::service('file_system')->realpath(\Drupal::service('module_handler')->getModule('tmgmt_cdt')->getPath() . '/json/version.json'));
            if (!$current) {
                throw new Exception('Cannot read local file: tmgmt_cdt/json/version.json');
            }
            $current = @Json::decode($current);
            $version['current'] = $current;
            unset($current);

            // store current version, before start online checks
            \Drupal::state()->set('tmgmt_cdt.version', $version);

            if ($only_current) {
                return $version;
            }

            $tags = self::curlGetFileContents('https://git.drupalcode.org/api/v4/projects/61708/repository/tags', $curl_options);
            if (!$tags) {
                throw new Exception('Cannot get module releases list');
            }
            $tags = @Json::decode($tags);

            $version['release'] = array(
                'name' => $tags[0]['name'],
                'created_at' => $tags[0]['commit']['created_at'],
                'commit' => $tags[0]['commit']['id'],
            );
            $version_release = self::curlGetFileContents('https://git.drupalcode.org/project/tmgmt_cdt/-/raw/' . $version['release']['commit'] . '/json/version.json', $curl_options);
            $version_release = @Json::decode($version_release);

            if (!$version_release) {
                $version['release'] = array();
            } else {
                $version['release'] = $version_release;
            }

            unset($version_release);
            unset($tags);

            $branch = self::curlGetFileContents('https://git.drupalcode.org/api/v4/projects/61708/repository/branches/8.x-2.x', $curl_options);
            if (!$branch) {
                throw new Exception('Cannot get module last updates from dev branch');
            }
            $branch = @Json::decode($branch);

            $version['dev'] = array(
                'name' => $branch['name'],
                'created_at' => $branch['commit']['created_at'],
                'commit' => $branch['commit']['id'],
            );
            $version_dev = self::curlGetFileContents('https://git.drupalcode.org/project/tmgmt_cdt/-/raw/' . $version['dev']['commit'] . '/json/version.json', $curl_options);

            $version_dev = @Json::decode($version_dev);

            if (!$version_dev) {
                $version['dev'] = aaray();
            } else {
                $version['dev'] = $version_dev;
            }

            unset($version_dev);
            unset($branch);

            \Drupal::state()->set('tmgmt_cdt.version', $version);

            return $version;

        } catch (\Exception $e) {
            // just a warning, not realy important
            \Drupal::messenger()->addWarning($e->getMessage());
            return false;
        }

    }

    /**
     * Get env mode and api_url.
     */
    public static function getEnvContext()
    {
        $info = array(
            'mode' => '',
            'ready' => '',
            'api_type' => '',
        );
        $info['mode'] = self::getTranslatorSetting('conf_mode.api_mode') === self::PRODUCTION
        ? t('Using the remote webservice (Connected)')
        : t('Local (Not connected)');

        $info['ready'] = self::getState('ready');

        if (self::getTranslatorSetting('conf_mode.api_mode') === self::PRODUCTION) {
            $info['api_type'] = self::getCredentials()['api_url'] === self::API_URL['PRODUCTION']
            ? t('Cdt <strong>Production</strong>: This job is real. <strong style="color:red">This is not a test!</strong>.')
            : t('Cdt <strong>Acceptance</strong>: Please contact the Cdt to process your test.');

        } else {
            $info['api_type'] = '<strong>Simulator</strong> engine for testing.';
        }
        return $info;

    }

    /**
     * calculate webservice json hash.
     */
    public static function webserviceHash()
    {
        try {
            $curl_options = array('CURLOPT_USERAGENT' => 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/101.0.0.0 Safari/537.36');

            $url = self::getCredentials()['api_url'] . 'swagger/docs/v1';
            $json_content = self::curlGetFileContents($url, $curl_options);
            return sha1($json_content);

        } catch (\Exception $e) {
            // just a warning, not realy important
            \Drupal::messenger()->addWarning($e->getMessage());
            return false;
        }

    }

}
