<?php

namespace Drupal\tmgmt_cdt;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\tmgmt\JobInterface;
use Drupal\tmgmt\TranslatorPluginUiBase;
use Drupal\tmgmt_cdt\TmgmtCdtCommands;
use Drupal\tmgmt_cdt\TmgmtCdtHelper;

/**
 * TmgmtCdtTranslatorUi.
 */
class TmgmtCdtTranslatorUi extends TranslatorPluginUiBase
{

    use MessengerTrait;
    use StringTranslationTrait;

    /**
     * {@inheritdoc}
     */
    public function checkoutInfo(JobInterface $job)
    {
        $form = [];
        try {

            TmgmtCdtHelper::showStatus(true);
            $job = TmgmtCdtHelper::jobAutoUpdate($job);

            $details = $this->t('Reference (request identifier): <strong>@cdt_request_identifier</strong><br>', ['@cdt_request_identifier' => $job->get('cdt_request_identifier')->value]);
            $details .= $this->t('Temporary id (correlation id): <strong>@cdt_correlation_id</strong><br>', ['@cdt_correlation_id' => $job->get('cdt_correlation_id')->value]);

            $api = TmgmtCdtApi::getInstance();
            $api->setParamsPath(TmgmtCdtHelper::splitRequestIdentifier($job->get('cdt_request_identifier')->value));
            $api->get('requests/{requestyear}/{requestnumber}');

            if (isset($api->response->content->status) && $api->response->status === 'success') {
                $details .= $this->t('CdT job status: <strong>@cdt_status</strong><br>', ['@cdt_status' => $api->response->content->status]);
            } else {
                $details .= $this->t('CdT job status: -<br>');
            }
            $form['job_information'] = [
                '#markup' => $details . '<br>',
            ];

            $is_on_pickup = $job->getCdtSetting('on_pickup', false);

            if ($job->isActive()) {

                if ($api->response->status === 'success' && ($api->response->content->status === TmgmtCdtHelper::API_STATUS_COMPLETED)) {
                    $this->messenger()->addStatus($this->t('Translations ready. Please click on the <strong>Pick up translations</strong> button 👇'));

                    $form['group_pull'] = [
                        '#type' => 'fieldset',
                        '#weight' => 10,
                        '#open' => true,
                    ];

                    $form['group_pull']['actions']['pull'] = [
                        '#type' => 'submit',
                        "#button_type" => "primary",
                        '#value' => $this->t('Pick up translations'),
                        '#submit' => [[$this, 'submitPullTranslations']],
                        '#weight' => -10,
                    ];

                } elseif ($api->response->status === 'success' && ($api->response->content->status === TmgmtCdtHelper::API_STATUS_QUOTED_PENDING_APPROVAL)) {
                    $this->messenger()->addStatus($this->t('Quotation ready. Please accept or reject the quotation 👇'));

                    $api->resetInstance();
                    $api->setParamsPath(TmgmtCdtHelper::splitRequestIdentifier($job->get('cdt_request_identifier')->value));
                    $api->get('requests/{requestyear}/{requestnumber}/quotationSummary');

                    if (isset($api->response->content->totalPrice) && $api->response->status === 'success') {

                        $form['group_quotation'] = [
                            '#type' => 'fieldset',
                            '#weight' => 10,
                            '#open' => true,
                        ];
                        $form['group_quotation']['details'] = [
                            '#markup' => $this->t('Total price: <strong style="font-size: 150%;" > @price </strong><br><br>', ['@price' => (int) $api->response->content->totalPrice]),
                        ];

                        $form['group_quotation']['actions']['accept_quotation'] = [
                            '#type' => 'submit',
                            "#button_type" => "primary",
                            '#value' => $this->t('Accept quotation'),
                            '#submit' => [[$this, 'submitAcceptQuotation']],
                            '#weight' => -9,
                        ];
                        $form['group_quotation']['actions']['cancel_quotation'] = [
                            '#type' => 'submit',
                            "#button_type" => "primary",
                            '#value' => $this->t('Cancel quotation'),
                            '#submit' => [[$this, 'submitCancelQuotation']],
                            '#weight' => -9,
                        ];
                        $form['group_quotation']['actions']['consent'] = [
                            '#type' => 'checkbox',
                            '#title' => $this->t('Yes I understand the implications of "accept quotation" and "cancel quotation" actions.'),
                            '#default_value' => 0,
                        ];
                    }

                } else {
                    $this->messenger()->addWarning($this->t('Translations or quotation not ready'));
                }

                $user = \Drupal::currentUser();
                if ($user->hasPermission('accept translation jobs') && $job->hasItemsToReview() && !$is_on_pickup) {
                    $form['group_accept'] = [
                        '#type' => 'fieldset',
                        '#weight' => 10,
                        '#open' => true,
                    ];
                    $form['group_accept']['actions']['accept_all_translation'] = [
                        '#type' => 'submit',
                        "#button_type" => "primary",
                        '#value' => $this->t('Accept all translations'),
                        '#submit' => [[$this, 'submitAcceptAllTranslations']],
                        '#weight' => -9,
                    ];
                    $form['group_accept']['actions']['consent'] = [
                        '#type' => 'checkbox',
                        '#title' => $this->t('Yes I understand that the translations will be automatically published without review <strong>(Not recommended)</strong>.'),
                        '#default_value' => 0,
                    ];
                }

            }

            return $form;

        } catch (\Exception $e) {

            $this->messenger()->addError($e->getMessage());

            return parent::checkoutInfo($job);
        }

    }

    /**
     * Submit button pull translations.
     */
    public function submitPullTranslations(array $form, FormStateInterface $form_state)
    {
        $this->messenger()->deleteAll();
        /**
         * @var \Drupal\tmgmt\TranslatorInterface $translator
         */
        $job = $form_state->getFormObject()->getEntity();
        TmgmtCdtHelper::pullTranslations($job);

    }

    /**
     * Submit button accept all translations.
     */
    public function submitAcceptAllTranslations(array $form, FormStateInterface $form_state)
    {

        $this->messenger()->deleteAll();

        if (!(bool) $form_state->getValues()['consent']) {
            $this->messenger()->addError($this->t('You forgot the checkbox "Yes I understand ..."'));
            return;
        }

        $job = $form_state->getFormObject()->getEntity();
        $job->acceptTranslation();

        if (tmgmt_job_check_finished($job->id())) {
            $job->setState(JobInterface::STATE_FINISHED);
            $job->set('target_language', 'multiple');
            $job->save();
            $job->addMessage($this->t('The translation job has been finished. (user action "Accept all translations"'));
        }

    }

    /**
     * Submit button accept quotation.
     */
    public function submitAcceptQuotation(array $form, FormStateInterface $form_state)
    {
        $this->messenger()->deleteAll();
        /**
         * @var \Drupal\tmgmt\TranslatorInterface $translator
         */
        $job = $form_state->getFormObject()->getEntity();

        if (!(bool) $form_state->getValues()['consent']) {
            $this->messenger()->addError($this->t('You forgot the checkbox "Yes I understand ..."'));
            return;
        }

        $api = TmgmtCdtApi::getInstance();
        $api->setParamsPath(TmgmtCdtHelper::splitRequestIdentifier($job->get('cdt_request_identifier')->value));
        $api->post('requests/{requestyear}/{requestnumber}/ApproveQuotation');

        if ($api->response->status === 'success') {
            $job->addMessage($this->t('The quotation has been accepted.'));
            $this->messenger()->addStatus($this->t('The quotation has been accepted.'));
        } else {
            $this->messenger()->addError($this->t('Error on processing the action'));
            $this->messenger()->addError(
                $this->t(
                    '@error',
                    ['@error' => implode(', ', $api->response->errors)]
                )
            );
        }

    }

    /**
     * Submit button accept quotation.
     */
    public function submitCancelQuotation(array $form, FormStateInterface $form_state)
    {
        $this->messenger()->deleteAll();
        /**
         * @var \Drupal\tmgmt\TranslatorInterface $translator
         */
        $job = $form_state->getFormObject()->getEntity();

        if (!(bool) $form_state->getValues()['consent']) {
            $this->messenger()->addError($this->t('You forgot the checkbox "Yes I understand ..."'));
            return;
        }

        $api = TmgmtCdtApi::getInstance();
        $api->setParamsPath(TmgmtCdtHelper::splitRequestIdentifier($job->get('cdt_request_identifier')->value));
        $api->post('requests/{requestyear}/{requestnumber}/CancelQuotation');

        if ($api->response->status === 'success') {
            $job->addMessage($this->t('The quotation has been cancelled.'));
            $this->messenger()->addStatus($this->t('The quotation has been cancelled.'));
        } else {
            $this->messenger()->addError($this->t('Error on processing the action'));
            $this->messenger()->addError(
                $this->t(
                    '@error',
                    ['@error' => implode(', ', $api->response->errors)]
                )
            );
        }

    }

    /**
     * {@inheritdoc}
     */
    public function buildConfigurationForm(array $form, FormStateInterface $form_state)
    {
        $form = parent::buildConfigurationForm($form, $form_state);

        TmgmtCdtHelper::showStatus();

        $this->messenger()->deleteAll();

        $form['conf_mode'] = [
            '#type' => 'details',
            '#title' => 'Environment selection',
            '#weight' => 10,
            '#open' => true,

        ];
        $form['conf_mode']['api_mode'] = [
            '#type' => 'select',
            '#default_value' => TmgmtCdtHelper::getTranslatorSetting('conf_mode.api_mode'),
            '#options' => [
                TmgmtCdtHelper::SIMULATION => $this->t('Simulation (Not connected)'),
                TmgmtCdtHelper::PRODUCTION => $this->t('Real (connected)'),
            ],

        ];

        $form['conf_credentials'] = [
            '#type' => 'details',
            '#title' => $this->t('Web service connection'),
            '#weight' => 20,

            '#open' => false,
        ];

        $credentials = TmgmtCdtHelper::getCredentials();
        if ($credentials['source'] === 'settings') {
            // from settings.php (cannot be edited)

            $api_url = array_flip(TmgmtCdtHelper::API_URL)[trim($credentials['api_url'])];
            $api_url = ucfirst(strtolower($api_url));

            $form['conf_credentials']['#description'] = $this->t('Credentials not editable on this instance.');
            $form['conf_credentials']['#description'] .= '<br>Url: <strong>' . $api_url . '</strong>';
            $form['conf_credentials']['#description'] .= '<br>Username: <strong>' . $credentials['api_username'] . '</strong>';
            $form['conf_credentials']['#description'] .= '<br>Password: <strong>' . str_repeat("*", strlen(\base64_decode($credentials['api_username']))) . '</strong>';

        } else {
            // from \Drupal::state (can be edited)
            $clean_label = function ($label) {
                return ucfirst(strtolower($label));
            };

            $form['conf_credentials']['api_url'] = [
                '#type' => 'select',
                '#title' => $this->t('Web service URL'),
                '#default_value' => $credentials['api_url'],

                '#description' => $this->t('For info <i>Acceptance: @acc_url, Production: @prod_url</i> .',
                    array(
                        '@acc_url' => TmgmtCdtHelper::API_URL['ACCEPTANCE'],
                        '@prod_url' => TmgmtCdtHelper::API_URL['PRODUCTION'],
                    )),
                '#options' => array_map($clean_label, array_flip(TmgmtCdtHelper::API_URL)),
            ];

            $form['conf_credentials']['api_username'] = [
                '#type' => 'textfield',
                '#title' => 'API username',
                '#default_value' => $credentials['api_username'],
                '#size' => 30,
                '#maxlength' => 50,
            ];
            $form['conf_credentials']['api_password'] = [
                '#type' => 'password',
                '#title' => $this->t('API password'),
                '#size' => 30,
                '#maxlength' => 50,
            ];
        }

        $form['conf_features'] = [
            '#type' => 'details',
            '#title' => $this->t('Features'),
            '#weight' => 30,
            '#open' => false,
        ];
        $form['conf_features']['convert_content'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Enable conversions. Protect specials contents inside <i>&lt;tmgmt-cdt&gt;</i> tags to avoid translation. <strong>(Strongly recommended)</strong>'),
            '#description' => $this->t('Disabling this option will disable all other conversion options below.'),
            '#default_value' => (bool) TmgmtCdtHelper::getTranslatorSetting('conf_features.convert_content'),
        ];
        $form['conf_features']['convert_html_script'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Enable html script tag conversion <i>&lt;script&gt;</i>. <strong>(Strongly recommended)</strong>'),
            '#default_value' => (bool) TmgmtCdtHelper::getTranslatorSetting('conf_features.convert_html_script'),
        ];
        $form['conf_features']['convert_drupal_token'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Enable Drupal tokens conversion: <i>[node:field_tags:entity:name], [current-page:title] .. etc</i>. <strong>(Strongly recommended)</strong>'),
            '#default_value' => (bool) TmgmtCdtHelper::getTranslatorSetting('conf_features.convert_drupal_token'),
        ];
        $form['conf_features']['convert_drupal_twig'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Enable Drupal twig conversion: <i>{{ title }}, {{ filed_custom }} .. etc</i>. <strong>(Strongly recommended)</strong>'),
            '#default_value' => (bool) TmgmtCdtHelper::getTranslatorSetting('conf_features.convert_drupal_twig'),
        ];
        $form['conf_features']['convert_custom'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Enable custom keywords conversion: <i>from the custom list</i>. <strong>(Strongly recommended)</strong>'),
            '#description' => $this->t('Works with content from <strong>Advanced > Custom Keywords</strong> option.'),
            '#default_value' => (bool) TmgmtCdtHelper::getTranslatorSetting('conf_features.convert_custom'),
        ];
        $form['conf_features']['html_tags_comparison'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Enable html tags comparison. <strong>(Strongly recommended)</strong>'),
            '#description' => $this->t('Compares the number of html tags between the source and the translation. If there is a difference, an alert message will be generated, but the import of translations will not be blocked. Useful for the reviews.'),
            '#default_value' => (bool) TmgmtCdtHelper::getTranslatorSetting('conf_features.html_tags_comparison'),
        ];
        $form['conf_features']['show_quotation'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Show quotation option in checkout form. <strong>(Not recommended)</strong>'),
            '#description' => $this->t('Please contact the @cdt before activating this option.', ['@cdt' => TmgmtCdtHelper::NAME]),
            '#default_value' => (bool) TmgmtCdtHelper::getTranslatorSetting('conf_features.show_quotation'),
        ];
        $form['conf_features']['disable_simulator_injection'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Disable simulator code injection (language code). <strong>(Not recommended)</strong>'),
            '#description' => $this->t('The translation proposed by the simulator will be identical to the source content.'),
            '#default_value' => (bool) TmgmtCdtHelper::getTranslatorSetting('conf_features.disable_simulator_injection'),
        ];
        $form['conf_features']['disable_xsd_validation'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Disable XSD schema validation. <strong>(Not recommended)</strong>'),
            '#description' => $this->t('The xml will be loaded without XSD schema validation. Can be useful if non-essential part of the xml does not pass validation.'),
            '#default_value' => (bool) TmgmtCdtHelper::getTranslatorSetting('conf_features.disable_xsd_validation'),
        ];
        $form['conf_features']['debug'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Log debugging information for API calls.'),
            '#description' => $this->t('The result of the API call is send to \Drupal::logger. 🚧 For dev/testing.'),
            '#default_value' => (bool) TmgmtCdtHelper::getTranslatorSetting('conf_features.debug'),
        ];
        $form['conf_features']['cart_field_convert'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Enable the conversion choice on each item cart field.'),
            '#description' => $this->t('Be able to disable conversion in a field containing conversions. 🚧 For dev/testing.'),
            '#default_value' => (bool) TmgmtCdtHelper::getTranslatorSetting('conf_features.cart_field_convert'),
        ];
        $form['conf_features']['fields_translation_mode'] = [
            '#type' => 'select',
            '#options' => TmgmtCdtHelper::FIELDS_TRANSLATION_MODE,
            '#title' => $this->t('When a cart item is not personalized, which fields selection mode to use.'),
            '#description' => $this->t('Does not apply when you manually edit your item fields in cart. (<strong>Automatic is strongly recommended</strong>)'),
            '#default_value' => TmgmtCdtHelper::getTranslatorSetting('conf_features.fields_translation_mode'),
        ];

        $form['conf_advanced'] = [
            '#type' => 'details',
            '#title' => $this->t('Advanced'),
            '#weight' => 30,
            '#open' => false,
        ];
        $form['conf_advanced']['custom_keywords'] = [
            '#type' => 'textarea',
            '#title' => $this->t('Custom Keywords. (Semicolon separated values, no space between elements)'),
            '#description' => $this->t('Keywords to protect by using conversion. <strong>Example:</strong> <i>&lt;none&gt;;&lt;front&gt;;_default;_none</i>. Used by the <strong>Features > Enable custom keywords conversion</strong> option.'),
            '#default_value' => TmgmtCdtHelper::getTranslatorSetting('conf_advanced.custom_keywords'),
            '#cols' => 50,
            '#rows' => 3,
        ];
        $form['conf_advanced']['max_pages_per_request'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Maximum number of pages per request'),
            '#description' => $this->t('If you change this option to more than 10, please contact the @cdt.', ['@cdt' => TmgmtCdtHelper::NAME]),
            '#default_value' => (int) TmgmtCdtHelper::getTranslatorSetting('conf_advanced.max_pages_per_request'),
            '#size' => 5,
        ];
        $form['conf_advanced']['xsd_link'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Default XSD Schema link.'),
            '#default_value' => TmgmtCdtHelper::getTranslatorSetting('conf_advanced.xsd_link'),
            '#size' => 100,
            '#maxlength' => 255,
        ];
        $form['conf_advanced']['exclude_fields'] = [
            '#type' => 'textarea',
            '#title' => $this->t('Fields name to exclude. (Semicolon separated values, no space between elements)'),
            '#description' => $this->t('Use field reference (see cart item edition). You can use the joker *. <strong>Example:</strong> <i>view][label;view][description;view][display][default][display_options][fields]*[separator</i>'),
            '#default_value' => TmgmtCdtHelper::getTranslatorSetting('conf_advanced.exclude_fields'),
            '#cols' => 50,
            '#rows' => 3,
        ];
        $form['conf_advanced']['job_max_items'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Maximum number of items for a job'),
            '#description' => $this->t('This prevents a Drupal crash due to too long operation'),
            '#default_value' => (int) TmgmtCdtHelper::getTranslatorSetting('conf_advanced.job_max_items'),
            '#size' => 5,
        ];
        $form['conf_advanced']['job_max_process'] = [
            '#type' => 'textfield',
            '#title' => $this->t('maximum number of items to process in a job'),
            '#description' => $this->t('For "pickup translations" and "accept all translations" buttons. You will have to do several clicks to complete the operation. This prevents a Drupal crash due to too long operation'),
            '#default_value' => (int) TmgmtCdtHelper::getTranslatorSetting('conf_advanced.job_max_process'),
            '#size' => 5,
        ];

        $form['conf_network'] = [
            '#type' => 'details',
            '#title' => $this->t('Network'),
            '#weight' => 40,
            '#open' => false,
        ];
        $form['conf_network']['curl_proxy_host'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Proxy host'),
            '#default_value' => TmgmtCdtHelper::getTranslatorSetting('conf_network.curl_proxy_host'),
            '#size' => 50,
            '#maxlength' => 100,
        ];
        $form['conf_network']['curl_proxy_port'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Proxy port'),
            '#default_value' => (int) TmgmtCdtHelper::getTranslatorSetting('conf_network.curl_proxy_port'),
            '#size' => 5,
            '#maxlength' => 10,
        ];
        $form['conf_network']['curl_proxy_user'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Proxy username'),
            '#default_value' => TmgmtCdtHelper::getTranslatorSetting('conf_network.curl_proxy_user'),
            '#size' => 30,
            '#maxlength' => 50,
        ];
        $form['conf_network']['curl_proxy_pass'] = [
            '#type' => 'password',
            '#title' => $this->t('Proxy password'),
            '#default_value' => TmgmtCdtHelper::getTranslatorSetting('conf_network.curl_proxy_pass'),
            '#size' => 30,
            '#maxlength' => 50,
        ];
        $form['conf_network']['curl_proxy_user_agent'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Proxy user agent'),
            '#default_value' => TmgmtCdtHelper::getTranslatorSetting('conf_network.curl_proxy_user_agent'),
            '#size' => 100,
            '#maxlength' => 255,
        ];
        $form['conf_network']['curl_options'] = [
            '#type' => 'textarea',
            '#title' => $this->t('CURL options (JSON format).'),
            '#default_value' => TmgmtCdtHelper::getTranslatorSetting('conf_network.curl_options'),
            '#cols' => 50,
            '#rows' => 5,
        ];

        $form['conf_commands'] = [
            '#type' => 'details',
            '#title' => $this->t('Command'),
            '#description' => $this->t('For maintenance and technical assistance. Please contact the @cdt for the use of this tool.', ['@cdt' => TmgmtCdtHelper::NAME]),
            '#weight' => 50,
            '#open' => false,
        ];
        $form['conf_commands']['command'] = [
            '#type' => 'select',
            '#options' => [
                'select' => $this->t('Select a command ...'),
                'customCommand' => $this->t('Run a custom command.'),
                'resetSettings' => $this->t('Reset all values <strong>(All your custom options will be deleted)</strong>.'),
                'proxySettings' => $this->t('Show Drupal proxy settings.'),
                'report' => $this->t('Generate a report of your TMGMT environment.'),
            ],
        ];
        $form['conf_commands']['custom'] = [
            '#type' => 'textarea',
            '#title' => $this->t('Custom Command. Instructions in Json format.'),
            '#default_value' => '',
            '#cols' => 100,
            '#rows' => 20,
            '#states' => array(
                'visible' => array(
                    ':input[name="settings[conf_commands][command]"]' => array('value' => 'customCommand'),
                ),
            ),
        ];

        $form['conf_commands']['actions']['execute'] = [
            '#type' => 'submit',
            "#button_type" => "primary",
            '#value' => $this->t('Execute command'),
        ];

        $form['conf_commands']['validate_command'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Yes I understand what I\'m doing, run the command.'),
            '#default_value' => false,
        ];

        $form['#array_parents'] = ['conf_mode', 'conf_advanced', 'conf_network'];

        return $form;
    }

    /**
     * {@inheritdoc}
     */
    public function validateConfigurationForm(array &$form, FormStateInterface $form_state)
    {
        $conf_commands = $form_state->getValues()['settings']['conf_commands'];
        $conf_credentials = $form_state->getValues()['settings']['conf_credentials'];
        $form_state->unsetValue(['settings', 'conf_credentials']);
        $form_state->unsetValue(['settings', 'conf_commands']);

        $settings = $form_state->getValues()['settings'];
        parent::validateConfigurationForm($form, $form_state);

        $this->messenger()->deleteAll();

        if (($conf_commands['command'] !== 'select') && !((bool) $conf_commands['validate_command'])) {
            $this->messenger()->addWarning($this->t('You forgot to check "Yes I understand ...'));
            return;
        }

        if (($conf_commands['command'] === 'select') && ((bool) $conf_commands['validate_command'])) {
            $this->messenger()->addWarning($this->t('You forgot to select a command.'));
            return;
        }

        if ((bool) $conf_commands['validate_command']) {
            if ($conf_commands['command'] === 'customCommand') {
                return TmgmtCdtCommands::command(@Json::decode($conf_commands['custom']));
            }

            return TmgmtCdtCommands::command(
                array(
                    'command' => $conf_commands['command'],
                    'data' => array(),
                )
            );

        }

        $is_ready = true;

        try {

            // Reset.
            \Drupal::state()->set('tmgmt_cdt.ready', false);
            \Drupal::state()->set('tmgmt_cdt.business_reference_data', new \stdClass());
            \Drupal::state()->set('tmgmt_cdt.api_access_token_expire', 0);
            \Drupal::state()->set('tmgmt_cdt.api_access_token', '');

            // save selected env
            $config = \Drupal::service('config.factory')->getEditable('tmgmt.translator.tmgmt_cdt');
            $config->set('settings.conf_mode.api_mode', $settings['conf_mode']['api_mode']);
            $config->save();

            // Webservice.
            $credentials = TmgmtCdtHelper::getCredentials();
            if ($credentials['source'] !== 'settings') {
                \Drupal::state()->set('tmgmt_cdt.api_url', @trim($conf_credentials['api_url']));
                \Drupal::state()->set('tmgmt_cdt.api_username', @trim($conf_credentials['api_username']));
                if (mb_strlen(@trim($conf_credentials['api_password'])) > 0) {
                    \Drupal::state()->set('tmgmt_cdt.api_password', base64_encode(@trim($conf_credentials['api_password'])));
                }
            }

            // Advanced.

            if (TmgmtCdtHelper::getTranslatorSetting('conf_mode.api_mode') === TmgmtCdtHelper::SIMULATION) {
                $relative_path = \Drupal::service('module_handler')->getModule('tmgmt_cdt')->getPath() . '/xml/schema.xsd';
                $full_path = \Drupal::service('file_system')->realpath($relative_path);
                $xml_contents = \file_get_contents($full_path);
                \Drupal::state()->set('tmgmt_cdt.xsd_source', $xml_contents);
            } else {
                if (!empty(trim($settings['conf_advanced']['xsd_link']))) {
                    $schemaSource = TmgmtCdtHelper::curlGetFileContents(trim($settings['conf_advanced']['xsd_link']));
                    if ($schemaSource !== false) {
                        \Drupal::state()->set('tmgmt_cdt.xsd_source', $schemaSource);
                    } else {
                        \Drupal::state()->set('tmgmt_cdt.xsd_source', '');
                    }
                } else {
                    \Drupal::state()->set('tmgmt_cdt.xsd_source', '');
                }

            }

            // Network.
            if (!TmgmtCdtHelper::isJson(trim($settings['conf_network']['curl_options']))) {
                throw new \Exception('Wrong JSON format for Curl options');
            }

            // Check part.
            $this->messenger()->addStatus($this->t('Check <strong>@name</strong> translator settings:', ['@name' => TmgmtCdtHelper::NAME]));
            $mode = TmgmtCdtHelper::getTranslatorSetting('conf_mode.api_mode') === TmgmtCdtHelper::SIMULATION ? $this->t('Simulation (Not connected)') : $this->t('Real (connected)');
            $this->messenger()->addStatus($this->t('Current environment: <strong>@mode</strong>.', ['@mode' => $mode]));

            if (TmgmtCdtHelper::getTranslatorSetting('conf_mode.api_mode') === TmgmtCdtHelper::PRODUCTION) {
                $clean_label = function ($label) {
                    return ucfirst(strtolower($label));
                };
                $api_url_label = array_map($clean_label, array_flip(TmgmtCdtHelper::API_URL))[trim(@$credentials['api_url'])];
                $this->messenger()->addStatus($this->t('Current webservice URL: <strong>@api_url_label</strong>.', ['@api_url_label' => $api_url_label]));
            }

            $api = TmgmtCdtApi::getInstance();
            $api->setReady(true);

            // Check webservice availability.
            if ($api->isWebserviceAvailable() === true) {
                $this->messenger()->addStatus($this->t('Webservice availability: <strong>OK</strong>.'));
            } else {
                $this->messenger()->addError(
                    $this->t(
                        'Webservice availability: <strong>FAILED</strong>.')
                );
                return;
            }

            // Check connectivity to webservice.
            if ($api->checkConnection() === true) {
                $this->messenger()->addStatus($this->t('Connection to webservice: <strong>OK</strong>.'));
            } else {
                $this->messenger()->addError(
                    $this->t(
                        'Connection to webservice: <strong>FAILED</strong>. @error',
                        ['@error' => implode(', ', $api->response->errors)]
                    )
                );
                return;
            }

            // check version
            if (TmgmtCdtHelper::getTranslatorSetting('conf_mode.api_mode') === TmgmtCdtHelper::SIMULATION) {
                $version = TmgmtCdtHelper::updateVersion(true);
                $this->messenger()->addStatus($this->t('Module version: <strong>@version</strong>.',
                    array(
                        '@version' => isset($version['current']['version']) ? $version['current']['version'] : 'unknown',
                    )));
            } else {
                $version = TmgmtCdtHelper::updateVersion();
                $this->messenger()->addStatus($this->t('Module version: current: <strong>@current</strong>, last release: <strong>@release</strong>, last dev: <strong>@dev</strong>, minimum required: <strong>@minimum</strong>.',
                    array(
                        '@current' => isset($version['current']['version']) ? $version['current']['version'] : 'unknown',
                        '@release' => isset($version['release']['version']) ? $version['release']['version'] : 'unknown',
                        '@dev' => isset($version['dev']['version']) ? $version['dev']['version'] : 'unknown',
                        '@minimum' => isset($version['dev']['minimum']) ? $version['dev']['minimum'] : 'unknown',
                    )));

                if ($version['dev']['minimum'] === 'unknown') {
                    $this->messenger()->addWarning($this->t('Module version: <strong>Cannot check minimum version</strong>. 🚧  Please check the current version online or contact the @cdt.',
                        array(
                            '@cdt' => TmgmtCdtHelper::NAME,
                        )));
                }

                if ((int) $version['dev']['minimum'] > (int) $version['current']['version']) {
                    $this->messenger()->addError($this->t('Module version: <strong>This version is too old, please update asap.</strong>. If you wish to use this outdated version, please contact the @cdt.',
                        array(
                            '@cdt' => TmgmtCdtHelper::NAME,
                        )));
                }
            }

            // check webservice hash version
            if (TmgmtCdtHelper::getTranslatorSetting('conf_mode.api_mode') !== TmgmtCdtHelper::SIMULATION) {
                $webservice_hash = TmgmtCdtHelper::webserviceHash();
                if (!$webservice_hash) {
                    $this->messenger()->addWarning($this->t('Webservice version: Cannot check version.'));
                } else {
                    $current_webservice_hash = $version['current']['apiJsonSha1'];
                    if ($webservice_hash !== $current_webservice_hash) {
                        $this->messenger()->addWarning($this->t('Webservice version: the webservice has evolved. Please contact the @cdt to check the compatibility.',
                            array(
                                '@cdt' => TmgmtCdtHelper::NAME,
                            )
                        ));
                    } else {
                        $this->messenger()->addStatus($this->t('Webservice version: <strong>OK</strong>.'));
                    }
                }

            }

            // Check languages list.
            if (\count(\Drupal::languageManager()->getLanguages()) > 1) {
                $this->messenger()->addStatus($this->t('Language list: <strong>OK</strong>.'));
            } else {
                $is_ready = false;
                $this->messenger()->addError($this->t('Language list: <strong>FAILED</strong>. @error', ['@error' => 'There must be at least 2 active languages.']));
            }

            // Check default language position.
            $defaut_lang = \Drupal::languageManager()->getDefaultLanguage()->getName();
            $languages = \Drupal::languageManager()->getLanguages();
            $first_lang = reset($languages)->getName();
            if ($first_lang === $defaut_lang) {
                $this->messenger()->addStatus($this->t('Default language position: <strong>OK</strong>.'));
            } else {
                $is_ready = false;
                $this->messenger()->addError($this->t('Default language position: <strong>FAILED</strong>. Please move the default language (@defaut) to the first position of the language list.', ['@defaut' => $defaut_lang]));
            }

            // Get business reference data.
            $api->resetInstance();
            $api->setReady(true);
            $api->get('requests/businessReferenceData');

            if ($api->response->status === 'success') {
                \Drupal::state()->set('tmgmt_cdt.business_reference_data', $api->response->content);
                $this->messenger()->addStatus($this->t('Business reference data: <strong>OK</strong>.'));
            } else {
                $is_ready = false;
                $this->messenger()->addError(
                    $this->t(
                        'Business reference data: <strong>FAILED</strong>. @error',
                        ['@error' => implode(', ', $api->response->errors)]
                    ),
                    'error', false
                );
            }

            $json_file = \Drupal::service('file_system')->realpath(\Drupal::service('module_handler')->getModule('tmgmt_cdt')->getPath() . '/json/fakeRequest.json');
            $fake_request = @\file_get_contents($json_file);
            if (!$fake_request) {
                $is_ready = false;
                $this->messenger()->addError(
                    $this->t(
                        'Cannot read the file: tmgmt_cdt/json/fakeRequest.json'
                    ),
                    'error', false
                );
            }

            $fake_request = @\json_decode($fake_request);
            if (!$fake_request) {
                $is_ready = false;
                $this->messenger()->addError(
                    $this->t(
                        'Error decoding json: tmgmt_cdt/json/fakeRequest.json'
                    ),
                    'error', false
                );
            }

            $business_reference_data = TmgmtCdtHelper::getState('business_reference_data');
            $fake_request->departmentCode = $business_reference_data->departments[0]->code;
            $fake_request->contactUserNames = [$business_reference_data->contacts[0]->userName];
            $fake_request->deliveryContactUsernames = [$business_reference_data->contacts[0]->userName];

            $api->resetInstance();
            $api->setReady(true);
            $api = TmgmtCdtApi::getInstance();

            if ($api->validateRequest($fake_request)) {
                $this->messenger()->addStatus($this->t('Webservice user has permission to send a job: <strong>OK</strong>.'));
            } else {
                $is_ready = false;
                $this->messenger()->addError(
                    $this->t(
                        'Webservice user has permission to send a job: <strong>FAILED</strong>. @error',
                        ['@error' => implode(', ', $api->response->errors)]
                    ),
                    'error', false
                );
            }

            // Check languages mapping.
            $error_languages = [];
            $api_languages = @(array) TmgmtCdtHelper::getState('business_reference_data')->languages;

            /**
             * @var \Drupal\tmgmt\TranslatorInterface $translator
             */
            $translator = $form_state->getFormObject()->getEntity();

            foreach (\Drupal::languageManager()->getLanguages() as $lang) {
                if (!in_array($translator->mapToRemoteLanguage($lang->getId()), $api_languages)) {
                    $error_languages[] = $lang->getId();
                }
            }

            if (\count($error_languages) > 0) {
                $is_ready = false;
                $this->messenger()->addError(
                    $this->t(
                        'Languages mapping: <strong>FAILED</strong>. These languages must be mapped to the web service: @langs.', [
                            '@langs' => implode(', ', $error_languages),
                        ]
                    )
                );
            } else {
                $this->messenger()->addStatus($this->t('Languages mapping: <strong>OK</strong>.'));
            }

            // Check translators list.
            $query = \Drupal::entityQuery('tmgmt_translator')
                ->condition('plugin', 'tmgmt_cdt', '<>');
            $result = $query->execute();

            // Warning about tmgmt modules
            if (!\Drupal::moduleHandler()->moduleExists('tmgmt_content')) {
                $this->messenger()->addWarning($this->t('Module <strong>Content Entity Source</strong> (tmgmt_content) is not installed. It is useful for translating content like articles, pages, taxonomy terms ... etc'));
            }
            if (!\Drupal::moduleHandler()->moduleExists('tmgmt_locale')) {
                $this->messenger()->addWarning($this->t('Module <strong>Locales Source</strong> (tmgmt_locale) is not installed. It is useful for translating user interface.'));
            }
            if (!\Drupal::moduleHandler()->moduleExists('tmgmt_config')) {
                $this->messenger()->addWarning($this->t('Module <strong>Config Entity Source</strong> (tmgmt_config) is not installed. It is useful for translating Views, Webforms, Blocks ... etc'));
            }

            if (!empty($result)) {
                $is_ready = false;
                $this->messenger()->addError($this->t('Translators list: <strong>FAILED</strong>. Please delete the other translators.'), false);
            } else {
                $this->messenger()->addStatus($this->t('Translators list: <strong>OK</strong>.'));
            }

            if (!$is_ready) {
                throw new \Exception('Errors detected. please correct your configuration according to the messages.');
            }

            \Drupal::state()->set('tmgmt_cdt.ready', true);
        } catch (\Exception $e) {
            \Drupal::state()->set('tmgmt_cdt.ready', false);
            $this->messenger()->addError($e->getMessage());
        }
        TmgmtCdtHelper::showStatus(true);

    }

    /**
     * {@inheritdoc}
     */
    public function checkoutSettingsForm(array $form, FormStateInterface $form_state, JobInterface $job)
    {

        $job_settings = [];
        $errors = 0;

        $user_cdt_default = @Json::decode(\Drupal::service('user.data')->get('tmgmt_cdt', \Drupal::currentUser()->id(), 'cdt_default'));

        if (!$user_cdt_default) {
            $user_cdt_default = [];
        }

        // Load job settings if existe, if not load default user settings.
        if ($job->getSetting('cdt')) {
            $job_settings['cdt'] = $job->getSetting('cdt');
        } else {
            if (\count($user_cdt_default) > 0) {
                $job_settings['cdt'] = $user_cdt_default;
                unset($job_settings['cdt']['related_jobs']);
            }
        }

        $business_reference_data = TmgmtCdtHelper::getState('business_reference_data');
        if (empty((array) $business_reference_data)) {
            $this->messenger()->addError($this->t('Cannot get Business Reference Data from <strong>@name</strong>', ['@name' => TmgmtCdtHelper::NAME]));
            $errors++;
        }

        if ((bool) TmgmtCdtHelper::getTranslatorSetting('conf_features.show_quotation')) {

            $form['cdt']['request_type'] = [
                '#type' => 'select',
                '#title' => $this->t('Request type'),
                '#options' => [
                    'TRANSLATION' => $this->t('Translation'),
                    'QUOTATION' => $this->t('Quotation'),
                ],
                '#required' => true,
                '#description' => $this->t('<i>Please note that the processing of <strong>QUOTATION</strong> requests requires an additional processing time depending on the volume and type of content. In case of doubt, please contact the CdT.</i>'),
            ];

        } else {
            $form['cdt']['request_type'] = [
                '#type' => 'hidden',
                '#value' => 'TRANSLATION',
            ];
        }

        $form['cdt']['target_languages'] = [
            '#type' => 'select',
            '#multiple' => true,
            '#title' => $this->t('Target languages'),
            '#chosen' => true,
            '#options' => tmgmt_available_languages([\Drupal::languageManager()->getDefaultLanguage()->getId()]),
            '#default_value' => isset($job_settings['cdt']['target_languages']) ? $job_settings['cdt']['target_languages'] : null,
            '#required' => true,
            '#description' => '<a class="button chosen-toggle select">' . $this->t('Select all') . '</a> <a class="button chosen-toggle deselect">' . $this->t('Deselect all') . '</a>' . $this->t('Select target language(s) for the request'),
        ];
        $form['#attached']['library'][] = 'tmgmt_cdt/checkout';
        $priority_list = [];
        if (!isset($business_reference_data->priorities)) {
            $this->messenger()->addError($this->t('Cannot find the priorities list'));
            $errors++;
        }
        foreach ($business_reference_data->priorities as $priority) {
            $priority_list[$priority->code] = $priority->description;
        }
        $form['cdt']['priority'] = [
            '#type' => 'select',
            '#title' => $this->t('Priority'),
            '#options' => $priority_list,
            '#default_value' => isset($job_settings['cdt']['priority']) ? $job_settings['cdt']['priority'] : null,
            '#required' => true,
            '#description' => $this->t('Select priority for the request'),
        ];
        unset($priority_list);

        $confidentiality_list = [];
        if (!isset($business_reference_data->confidentialities)) {
            $this->messenger()->addError($this->t('Cannot find the confidentialities list'));
            $errors++;
        }
        foreach ($business_reference_data->confidentialities as $confidentiality) {
            $confidentiality_list[$confidentiality->code] = $confidentiality->description;
        }
        $form['cdt']['confidentiality'] = [
            '#type' => 'select',
            '#title' => $this->t('Confidentiality'),
            '#options' => $confidentiality_list,
            '#default_value' => isset($job_settings['cdt']['confidentiality']) ? $job_settings['cdt']['confidentiality'] : null,
            '#required' => true,
            '#description' => $this->t('Select confidentiality for the request'),
        ];
        unset($confidentiality_list);

        $department_list = [];
        if (!isset($business_reference_data->departments)) {
            $errors++;
        }
        foreach ($business_reference_data->departments as $department) {
            $department_list[$department->code] = $department->description;
        }
        $form['cdt']['department'] = [
            '#type' => 'select',
            '#title' => $this->t('Department'),
            '#options' => $department_list,
            '#default_value' => isset($job_settings['cdt']['department']) ? $job_settings['cdt']['department'] : null,
            '#required' => true,
            '#description' => $this->t('Select department for the request'),
        ];
        unset($department_list);

        $contact_list = [];
        if (!isset($business_reference_data->contacts)) {
            $this->messenger()->addError($this->t('Cannot find the contacts list'));
            $errors++;
        }
        foreach ($business_reference_data->contacts as $contact) {
            $contact_list[$contact->userName] = $contact->firstName . ' ' . $contact->lastName;
        }
        asort($contact_list);

        $form['cdt']['contacts'] = [
            '#type' => 'select',
            '#multiple' => true,
            '#title' => $this->t('Contacts'),
            '#chosen' => true,
            '#options' => $contact_list,
            '#default_value' => isset($job_settings['cdt']['contacts']) ? array_values($job_settings['cdt']['contacts']) : null,
            '#required' => true,
            '#description' => $this->t('Select contact(s) for the request'),
        ];

        $form['cdt']['deliver_to'] = [
            '#type' => 'select',
            '#multiple' => true,
            '#title' => $this->t('Deliver to'),
            '#chosen' => true,
            '#options' => $contact_list,
            '#default_value' => isset($job_settings['cdt']['deliver_to']) ? array_values($job_settings['cdt']['deliver_to']) : null,
            '#required' => true,
            '#description' => $this->t('Select recipient contact(s) for the request'),
        ];
        unset($contact_list);

        $form['cdt']['options'] = [
            '#type' => 'select',
            '#title' => $this->t('Translation options'),
            '#options' => TmgmtCdtHelper::API_COMMENT_URLS,
            '#default_value' => isset($job_settings['cdt']['options']) ? $job_settings['cdt']['options'] : null,
            '#required' => true,
            '#description' => $this->t('Select the option according to your needs'),
        ];

        $query = \Drupal::entityQuery('tmgmt_job')
            ->condition('translator', 'tmgmt_cdt')
            ->condition(
                'state', [JobInterface::STATE_ACTIVE,
                    JobInterface::STATE_FINISHED,
                ], 'IN'
            );
        $ids = $query->execute();

        $ref_jobs = \Drupal::entityTypeManager()->getStorage('tmgmt_job')->loadMultiple($ids);

        $related_jobs_list = [];
        foreach ($ref_jobs as $refJob) {
            $related_jobs_list[$refJob->get('cdt_request_identifier')->value] = $refJob->get('label')->value;
        }

        $form['cdt']['related_jobs'] = [
            '#type' => 'select',
            '#multiple' => true,
            '#title' => $this->t('Related jobs'),
            '#options' => $related_jobs_list,
            '#chosen' => true,
            '#default_value' => @\count($job_settings['cdt']['related_jobs']) > 1 ? array_values($job_settings['cdt']['related_jobs']) : -1,
            '#description' => $this->t('Select jobs linked to this one'),
        ];
        unset($related_jobs_list);

        $form['cdt']['phone'] = [
            '#type' => 'number',
            '#title' => $this->t('Phone'),
            '#default_value' => isset($job_settings['cdt']['phone']) ? $job_settings['cdt']['phone'] : '+0',
            '#size' => 20,
            '#maxlength' => 15,
            '#description' => $this->t('15 numeric characters maximum'),
            '#pattern' => '[^\\d]*',
        ];

        $form['cdt']['comments'] = [
            '#type' => 'textarea',
            '#title' => $this->t('Comments'),
            '#default_value' => isset($job_settings['cdt']['comments']) ? $job_settings['cdt']['comments'] : '',
            '#cols' => 30,
            '#rows' => 3,
        ];

        if ($errors === 0) {
            return parent::checkoutSettingsForm($form, $form_state, $job);
        } else {
            $this->messenger()->addError($this->t('Errors detected. please check the CdT configuration'));
        }

    }

}
